@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "bsd" -*- */
 /* armor.c - Armor filters
- *        Copyright (C) 2001, 2002 Timo Schulz
+ *        Copyright (C) 2001, 2002, 2003 Timo Schulz
  *        Copyright (C) 1998-2001 Free Software Foundation, Inc.
  *
  * This file is part of OpenCDK.
@@ -356,6 +356,12 @@ armor_encode( void * opaque, FILE * in, FILE * out )
 }
 
 
+/**
+ * cdk_armor_filter_use:
+ * @inp: the stream to check
+ *
+ * Check if the stream contains armored data.
+ **/
 int
 cdk_armor_filter_use (CDK_STREAM inp)
 {
@@ -425,7 +431,6 @@ armor_decode (void * opaque, FILE *in, FILE *out)
     }
 
     if (feof (in) || !pgp_data) {
-        printf( "NO DATA FOUND\n" );
         rc = CDK_Armor_Error; /* no data found */
         goto leave;
     }
@@ -508,11 +513,12 @@ armor_decode (void * opaque, FILE *in, FILE *out)
 
 
 /**
- * cdk_file_armor: Protect a file with ASCII armor.
+ * cdk_file_armor:
  * @hd: Handle
  * @file: Name of the file to protect.
  * @output: Output filename.
  *
+ * Protect a file with ASCII armor.
  **/
 int
 cdk_file_armor (CDK_HD hd, const char * file, const char * output)
@@ -549,10 +555,11 @@ cdk_file_armor (CDK_HD hd, const char * file, const char * output)
 
 
 /**
- * cdk_file_dearmor: Remove ASCII armor from a file.
+ * cdk_file_dearmor:
  * @file: Name of the file to unprotect.
  * @output: Output filename.
  *
+ * Remove ASCII armor from a file.
  **/
 int
 cdk_file_dearmor (const char * file, const char * output)
