@@ -74,7 +74,10 @@ struct cdk_handle_s {
     struct {
         Verify_Result verify;
     } result;
-
+    struct {
+        cdkPKT_secret_key * sk;
+        unsigned on:1;
+    } cache;
     CDK_DEK dek;
     CDK_S2K_HD s2k;
     struct {
