@@ -38,6 +38,7 @@ struct mainproc_ctx_s {
     CDK_DEK dek;
     CDK_STREAM s;
     CDK_KBNODE node;
+    CDK_STREAM tmpfp;
     struct {
         unsigned present:1;
         unsigned one_pass:1;
@@ -110,42 +111,42 @@ sym_stream_encrypt (CDK_HD hd, CDK_STREAM inp, CDK_STREAM out)
         goto start; /* skip the pkt_symkey_enc packet. */
     }
   
-    cdk_stream_set_cache (out, 1);
-    if (hd->opt.compat)
-        write_marker_packet (out);
+    cdk_stream_set_cache( out, 1 );
+    if( hd->opt.compat )
+        write_marker_packet( out );
       
-    enc = cdk_calloc (1, sizeof *enc);
-    if (!enc) {
+    enc = cdk_calloc( 1, sizeof *enc );
+    if( !enc ) {
         rc = CDK_Out_Of_Core;
         goto fail;
     }
     enc->version = 4;
     enc->cipher_algo = hd->dek->algo;
     enc->s2k = hd->s2k;
-    cdk_pkt_init (&pkt);
+    cdk_pkt_init( &pkt );
     pkt.pkttype = CDK_PKT_SYMKEY_ENC;
     pkt.pkt.symkey_enc = enc;
-    rc = cdk_pkt_build (out, &pkt);
-    cdk_free (enc);
+    rc = cdk_pkt_build( out, &pkt );
+    cdk_free( enc );
     if( rc )
         goto fail;
-    cdk_stream_set_cache (out, 0);
+    cdk_stream_set_cache( out, 0 );
 
  start:
-    if (hd->opt.armor)
-        cdk_stream_set_armor_flag (out, 0);
-    cdk_stream_set_cipher_flag (out, hd->dek, hd->opt.mdc);
+    if( hd->opt.armor )
+        cdk_stream_set_armor_flag( out, 0 );
+    cdk_stream_set_cipher_flag( out, hd->dek, hd->opt.mdc );
     if( hd->opt.compress )
         cdk_stream_set_compress_flag( out, hd->compress.algo,
                                       hd->compress.level );
-    cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
-    if (hd->opt.rfc1991)
-        plaintext_set_rfc1991 (out);
+    cdk_stream_set_literal_flag( out, 0, _cdk_stream_get_fname( inp ) );
+    if( hd->opt.rfc1991 )
+        plaintext_set_rfc1991( out );
 
-    rc = cdk_stream_kick_off (inp, out);
+    rc = cdk_stream_kick_off( inp, out );
 
- fail:
-    _cdk_passphrase_free (pw);
+fail:
+    _cdk_passphrase_free( pw );
     return rc;
 }
 
@@ -206,18 +207,18 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
     int use_rfc1991 = 0;
     int rc = 0;
 
-    if (!hd || !inp || !out)
+    if( !hd || !inp || !out )
         return CDK_Inv_Value;
 
-    if (!remusr)
-        return sym_stream_encrypt (hd, inp, out);
+    if( !remusr )
+        return sym_stream_encrypt( hd, inp, out );
 
-    rc = cdk_pklist_build (hd->db.pub, remusr, &pkl, GCRY_PK_USAGE_ENCR);
-    if (rc)
+    rc = cdk_pklist_build( hd->db.pub, remusr, &pkl, GCRY_PK_USAGE_ENCR );
+    if( rc )
         return rc;
 
-    use_rfc1991 = use_rfc1991_format (hd, pkl);
-    cipher_algo = select_cipher_algo (use_rfc1991, pkl);    
+    use_rfc1991 = use_rfc1991_format( hd, pkl );
+    cipher_algo = select_cipher_algo( use_rfc1991, pkl );
     cdk_dek_free( hd->dek );
     rc = cdk_dek_new( &hd->dek );
     if( !rc )
@@ -225,37 +226,36 @@ cdk_stream_encrypt( CDK_HD hd, CDK_STRLIST remusr,
     if( !rc )
         rc = cdk_dek_set_key( hd->dek, NULL, 0 );
     if( rc ) {
-        cdk_pklist_release (pkl);
+        cdk_pklist_release( pkl );
         return rc;
     }
-    compress_algo = use_rfc1991? CDK_COMPRESS_ZIP : hd->compress.algo;
+    compress_algo = use_rfc1991? CDK_COMPRESS_ZIP: hd->compress.algo;
 
-    if (!hd->opt.rfc1991 && !hd->opt.compat)
-        cdk_dek_set_mdc_flag( hd->dek, cdk_pklist_use_mdc (pkl) );
+    if( !hd->opt.rfc1991 && !hd->opt.compat )
+        cdk_dek_set_mdc_flag( hd->dek, cdk_pklist_use_mdc( pkl ) );
     hd->dek->rfc1991 = use_rfc1991;
 
-    cdk_stream_set_cache (out, 1);
-    if (hd->opt.compat && !hd->opt.rfc1991)
-        write_marker_packet (out);
-  
+    cdk_stream_set_cache( out, 1 );
+    if( hd->opt.compat && !hd->opt.rfc1991 )
+        write_marker_packet( out );
+    
     rc = cdk_pklist_encrypt( pkl, hd->dek, out );
     cdk_pklist_release( pkl );
     if( rc )
         return rc;
 
-    cdk_stream_set_cache (out, 0);
+    cdk_stream_set_cache( out, 0 );
 
-    if (hd->opt.armor)
-        cdk_stream_set_armor_flag (out, 0);
-    cdk_stream_set_cipher_flag (out, hd->dek, 0);
-    if (hd->opt.compress)
+    if( hd->opt.armor )
+        cdk_stream_set_armor_flag( out, 0 );
+    cdk_stream_set_cipher_flag( out, hd->dek, 0 );
+    if( hd->opt.compress )
         cdk_stream_set_compress_flag( out, compress_algo, hd->compress.level );
-    cdk_stream_set_literal_flag (out, 0, _cdk_stream_get_fname (inp));
-    if (hd->dek->rfc1991)
-        plaintext_set_rfc1991 (out);
+    cdk_stream_set_literal_flag( out, 0, _cdk_stream_get_fname( inp ) );
+    if( hd->dek->rfc1991 )
+        plaintext_set_rfc1991( out );
 
-    rc = cdk_stream_kick_off (inp, out);
-    return rc;
+    return cdk_stream_kick_off( inp, out );
 }
 
 
@@ -288,10 +288,10 @@ cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
 
 
 static void
-set_reset (CTX c, CDK_STREAM inp)
+set_reset( CTX c, CDK_STREAM inp )
 {
-    if (!c->reset) {
-        cdk_stream_control (inp, CDK_STREAMCTL_RESET, 1);
+    if( !c->reset ) {
+        cdk_stream_control( inp, CDK_STREAMCTL_RESET, 1 );
         c->reset = 1;
     }
 }
@@ -311,7 +311,7 @@ write_status (CDK_HD hd, int type, const char * fmt, ...)
     n = vasprintf (&buf, fmt, arg_ptr);
     buf[n] = '\0';
     hd->callback (hd->callback_value, type, buf);
-    free (buf);
+    _cdk_vasprintf_free (buf);
     va_end (arg_ptr);
 }
 
@@ -380,7 +380,28 @@ handle_symkey_enc (CTX c, CDK_PACKET * pkt)
 
 
 static int
-handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
+get_seckey( CDK_HD hd, CDK_KEYDB_HD db, u32 * keyid, cdkPKT_secret_key **r_sk )
+{
+    int rc;
+
+    if( hd->cache.on && hd->cache.sk ) {
+        cdkPKT_secret_key * sk = hd->cache.sk;
+        cdk_sk_get_keyid( sk, NULL );
+        if( sk->keyid[0] == keyid[0] && sk->keyid[1] == keyid[1] ) {
+            if( r_sk )
+                *r_sk = sk;
+            return 0;
+        }
+    }
+    rc = cdk_keydb_get_sk( db, keyid, r_sk );
+    if( hd->cache.on )
+        hd->cache.sk = *r_sk;
+    return rc;
+}
+    
+
+static int
+handle_pubkey_enc( CTX c, CDK_HD hd, CDK_PACKET * pkt )
 {
     cdkPKT_pubkey_enc * enc;
     cdkPKT_secret_key * sk = NULL;
@@ -401,10 +422,11 @@ handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
        skip this packet and continue without errors */
     if( cdk_keydb_check_sk( hd->db.sec, enc->keyid ) )
         return 0;
-    rc = cdk_keydb_get_sk( hd->db.sec, enc->keyid, &sk );
+    rc = get_seckey( hd, hd->db.sec, enc->keyid, &sk );
     if( !rc )
         rc = cdk_dek_extract( &c->dek, enc, sk );
-    _cdk_free_seckey( sk );
+    if( !hd->cache.on )
+        _cdk_free_seckey( sk );
     return rc;
 }
 
@@ -518,10 +540,14 @@ handle_plaintext( CTX c, CDK_PACKET * pkt, CDK_STREAM * ret_out )
     if( !ret_out )
         return CDK_Inv_Value;
 
-    s = c->output? c->output : pkt->pkt.plaintext->name;
-    rc = cdk_stream_create( s, ret_out );
-    if( rc )
-        return rc;
+    if( !c->tmpfp ) {
+        s = c->output? c->output : pkt->pkt.plaintext->name;
+        rc = cdk_stream_create( s, ret_out );
+        if( rc )
+            return rc;
+    }
+    else
+        *ret_out = c->tmpfp;
     out = *ret_out;
     pt = pkt->pkt.plaintext;
     cdk_stream_seek( c->s, c->sig.present? c->sig.pt_offset : 0 );
@@ -644,7 +670,7 @@ free_mainproc( CTX c )
 
 
 static int
-do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM *ret_out )
+do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM * ret_out )
 {
     CDK_PACKET * pkt = NULL;
     CDK_KBNODE n = NULL, node;
@@ -779,7 +805,8 @@ do_proc_packets( CDK_HD hd, CTX c, CDK_STREAM inp, CDK_STREAM *ret_out )
 
 
 int
-_cdk_proc_packets( CDK_HD hd, CDK_STREAM inp, const char * output,
+_cdk_proc_packets( CDK_HD hd, CDK_STREAM inp,
+                   const char * output, CDK_STREAM outstream,
                    GCRY_MD_HD md )
 {
     CDK_STREAM out = NULL;
@@ -788,16 +815,21 @@ _cdk_proc_packets( CDK_HD hd, CDK_STREAM inp, const char * output,
 
     if( !inp )
         return CDK_Inv_Value;
-  
+    if( output && outstream )
+        return CDK_Inv_Mode;
+
     c = cdk_calloc( 1, sizeof *c );
     if( !c )
         return CDK_Out_Of_Core;
     if( output )
         c->output = output;
+    if( outstream )
+        c->tmpfp = outstream;
     if( md )
         c->sig.md = md;
     rc = do_proc_packets( hd, c, inp, &out );
-    cdk_stream_close( out );
+    if( !c->tmpfp )
+        cdk_stream_close( out );
     free_mainproc( c );
     return rc;
 }
@@ -864,9 +896,128 @@ cdk_file_decrypt( CDK_HD hd, const char * file, const char * output )
     if( !rc )
         rc = check_pubkey_enc_list( inp, hd->db.sec );
     if( !rc )
-        rc = _cdk_proc_packets( hd, inp, output, NULL );
+        rc = _cdk_proc_packets( hd, inp, output, NULL, NULL );
 
     cdk_stream_close( inp );
     return rc;
 }
 
+
+int
+cdk_stream_decrypt( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out )
+{
+    int rc;
+
+    rc = check_pubkey_enc_list( inp, hd->db.sec );
+    if( !rc )
+        rc = _cdk_proc_packets( hd, inp, NULL, out, NULL );
+    return rc;
+}
+
+
+/**
+ * cdk_data_transform:
+ * @hd: session handle
+ * @mode: crypto mode
+ * @locusr: local user list (sign mode only)
+ * @remusr: remote users 'recipients'
+ * @inbuf: input buffer with data
+ * @insize: length of data in bytes
+ * @outbuf: pointer to the output data (will be allocated)
+ * @outsize: size of the new data in bytes
+ * @modval: value for the modus (for example sign mode)
+ *
+ * This function transforms data into the given openpgp mode. It works
+ * exactly like the cdk_file_xxx functions with the exception that it can
+ * be used with memory and not with streams or files.
+ **/
+int
+cdk_data_transform( CDK_HD hd, int mode,
+                    CDK_STRLIST locusr, CDK_STRLIST remusr,
+                    const void * inbuf, size_t insize,
+                    byte ** outbuf, size_t * outsize,
+                    int modval )
+{
+    CDK_STREAM inp, out;
+    CDK_KEYDB_HD db;
+    size_t nlen;
+    int rc, nbytes = 0;
+    
+    if( !hd )
+        return CDK_Inv_Value;
+    if( !mode )
+        return 0;
+    if( mode == CDK_CRYPTYPE_SIGN && !locusr )
+        return CDK_Inv_Value;
+    if( !inbuf || !insize || !outbuf )
+        return CDK_Inv_Value;
+
+    inp = cdk_stream_tmp_from_mem( inbuf, insize );
+    if( !inp )
+        return CDK_Out_Of_Core;
+    out = cdk_stream_tmp( );
+    if( !out ) {
+        cdk_stream_close( inp );
+        return CDK_Out_Of_Core;
+    }
+
+    cdk_stream_tmp_set_mode( inp, 0 );
+    cdk_stream_tmp_set_mode( out, 1 );
+        
+    switch( mode ) {
+    case CDK_CRYPTYPE_ENCRYPT:
+        rc = cdk_stream_encrypt( hd, remusr, inp, out );
+        break;
+
+    case CDK_CRYPTYPE_DECRYPT:
+        rc = cdk_stream_decrypt( hd, inp, out );
+        break;
+
+    case CDK_CRYPTYPE_SIGN:
+        rc = cdk_stream_sign( hd, inp, out, locusr, remusr, 0, modval );
+        break;
+
+    case CDK_CRYPTYPE_VERIFY:
+        rc = cdk_stream_verify( hd, inp, out );
+        break;
+
+    case CDK_CRYPTYPE_EXPORT:
+        if( cdk_handle_control( hd, CDK_CTLF_GET, CDK_CTL_ARMOR ) )
+            cdk_stream_set_armor_flag( out, CDK_ARMOR_PUBKEY );
+        db = cdk_handle_get_keydb( hd, CDK_DBTYPE_PK_KEYRING );
+        rc = cdk_keydb_export( db, out, remusr );
+        break;
+
+    default:
+        rc = CDK_Inv_Mode;
+        break;
+    }
+
+    cdk_stream_close( inp );
+    if( !rc && mode != CDK_CRYPTYPE_VERIFY ) {
+        rc = cdk_stream_seek( out, 0 );
+        if( !rc )
+            rc = cdk_stream_flush( out );
+        if( !rc ) {
+            nlen = cdk_stream_get_length( out );
+            if( nlen ) {
+                *outbuf = cdk_calloc( 1, nlen + 1 );
+                *outsize = nlen;
+                if( !rc ) {
+                    cdk_stream_tmp_set_mode( out, 0 );
+                    nbytes = cdk_stream_read( out, *outbuf, nlen );
+                }
+                if( nbytes == CDK_EOF )
+                    rc = CDK_General_Error;
+            }
+        }
+    }
+    else if( !rc && mode == CDK_CRYPTYPE_VERIFY ) {
+        *outbuf = cdk_sig_get_info_as_xml( hd, 0 );
+        *outsize = strlen( *outbuf );
+    }
+    cdk_stream_close( out );
+    return rc;
+}
+
+    
