@@ -1,6 +1,6 @@
 /* kbnode.c -  keyblock node utility functions
  *        Copyright (C) 1998-2001 Free Software Foundation, Inc.
- *        Copyright (C) 2002 Timo Schulz
+ *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -35,6 +35,12 @@
 #define is_cloned_kbnode(a)   ((a)->private_flag & 2)
 
 
+/**
+ * cdk_kbnode_new:
+ * @pkt: the packet to add
+ *
+ * Allocate a new key node and add the packet.
+ **/
 CDK_KBNODE
 cdk_kbnode_new( CDK_PACKET * pkt )
 {
@@ -55,6 +61,12 @@ _cdk_kbnode_clone( CDK_KBNODE node )
 }
 
 
+/**
+ * cdk_kbnode_release:
+ * @n: the key node
+ *
+ * Release the memory of the node.
+ **/
 void
 cdk_kbnode_release( CDK_KBNODE n )
 {
@@ -71,10 +83,12 @@ cdk_kbnode_release( CDK_KBNODE n )
 }
 
 
-/****************
- * Delete NODE.
- * Note: This only works with walk_kbnode!!
- */
+/**
+ * cdk_kbnode_delete:
+ * @node: the ke keynode.
+ *
+ * Delete @node.
+ **/
 void
 cdk_kbnode_delete( CDK_KBNODE node )
 {
@@ -97,21 +111,26 @@ _cdk_kbnode_add( CDK_KBNODE root, CDK_KBNODE node )
 }
 
 
-/****************
- * Insert NODE into the list after root but before a packet which is not of
- * type PKTTYPE (only if PKTTYPE != 0)
- */
+/**
+ * cdk_kbnode_insert:
+ * @root: the root key node
+ * @node: the node to add
+ * @pkttype: packet type
+ *
+ * Insert @node into the list after @root but before a packet which is not of
+ * type @pkttype (only if @pkttype != 0).
+ **/
 void
-cdk_kbnode_insert (CDK_KBNODE root, CDK_KBNODE node, int pkttype)
+cdk_kbnode_insert( CDK_KBNODE root, CDK_KBNODE node, int pkttype )
 {
-    if ( !pkttype ) {
+    if( !pkttype ) {
         node->next = root->next;
         root->next = node;
     }
     else {
         CDK_KBNODE n1;
         for( n1 = root; n1->next; n1 = n1->next )
-            if (pkttype != n1->next->pkt->pkttype ) {
+            if( pkttype != n1->next->pkt->pkttype ) {
                 node->next = n1->next;
                 n1->next = node;
                 return;
@@ -123,10 +142,15 @@ cdk_kbnode_insert (CDK_KBNODE root, CDK_KBNODE node, int pkttype)
 }
 
 
-/****************
- * Find the previous node (if PKTTYPE = 0) or the previous node
- * with pkttype PKTTYPE in the list starting with ROOT of NODE.
- */
+/**
+ * cdk_kbnode_find_prev:
+ * @root: the root key node
+ * @node: the key node
+ * @pkttype: packet type
+ *
+ * Find the previous node (if @pkttype = 0) or the previous node
+ * with pkttype @pkttype in the list starting with @root of @node.
+ **/
 CDK_KBNODE
 cdk_kbnode_find_prev( CDK_KBNODE root, CDK_KBNODE node, int pkttype )
 {
@@ -140,9 +164,13 @@ cdk_kbnode_find_prev( CDK_KBNODE root, CDK_KBNODE node, int pkttype )
 }
 
 
-/****************
+/**
+ * cdk_kbnode_find_next:
+ * @node: the key node
+ * @pkttype: packet type
+ *
  * Ditto, but find the next packet.  The behaviour is trivial if
- * PKTTYPE is 0 but if it is specified, the next node with a packet
+ * @pkttype is 0 but if it is specified, the next node with a packet
  * of this type is returned.  The function has some knowledge about
  * the valid ordering of packets: e.g. if the next signature packet
  * is requested, the function will not return one if it encounters
@@ -170,6 +198,13 @@ cdk_kbnode_find_next (CDK_KBNODE node, int pkttype)
 }
 
 
+/**
+ * cdk_kbnode_find:
+ * @node: the key node
+ * @pkttype: packet type
+ *
+ * Try to find the next node with the packettype @pkttype.
+ **/
 CDK_KBNODE
 cdk_kbnode_find (CDK_KBNODE node, int pkttype)
 {
@@ -181,6 +216,13 @@ cdk_kbnode_find (CDK_KBNODE node, int pkttype)
 }
 
 
+/**
+ * cdk_kbnode_find_packet:
+ * @node: the key node
+ * @pkttype: packet type
+ *
+ * Same as cdk_kbnode_find but it returns the packet instead of the node.
+ **/
 CDK_PACKET*
 cdk_kbnode_find_packet (CDK_KBNODE node, int pkttype)
 {
@@ -307,6 +349,12 @@ cdk_kbnode_move (CDK_KBNODE * root, CDK_KBNODE node, CDK_KBNODE where)
 }
 
 
+/**
+ * cdk_kbnode_get_packet:
+ * @node: the key node
+ *
+ * Return the packet which is stored inside the node in @node.
+ **/
 CDK_PACKET *
 cdk_kbnode_get_packet (CDK_KBNODE node)
 {
@@ -316,6 +364,14 @@ cdk_kbnode_get_packet (CDK_KBNODE node)
 }
 
 
+/**
+ * cdk_kbnode_read_from_mem:
+ * @ret_node: the new key node
+ * @buf: the buffer which stores the key sequence
+ * @buflen: the length of the buffer
+ *
+ * Try to read a key node from the memory buffer @buf.
+ **/
 int
 cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,const byte * buf,size_t buflen)
 {
@@ -341,6 +397,16 @@ cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,const byte * buf,size_t buflen)
 }
 
 
+/**
+ * cdk_kbnode_write_to_mem:
+ * @node: the key node
+ * @buf: the buffer to store the node data
+ * @r_nbytes: the new length of the buffer.
+ *
+ * Try to write the contents of the key node to the buffer @buf and
+ * return the length of it in @r_nbytes. If buff is zerok, only the
+ * length of the node is calculated and returned in @r_nbytes.
+ **/
 int
 cdk_kbnode_write_to_mem (CDK_KBNODE node, byte * buf, size_t * r_nbytes)
 {
