@@ -766,42 +766,53 @@ cdk_keydb_get_bypattern (CDK_KEYDB_HD hd, const char *patt,
 
 
 static int
-keydb_check_key (CDK_PACKET * pkt)
+keydb_check_key( CDK_PACKET * pkt )
 {
     cdkPKT_public_key * pk;
+    int is_sk = 0, valid = 0;
   
-    if (pkt->pkttype == CDK_PKT_PUBLIC_KEY
-        || pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
+    if( pkt->pkttype == CDK_PKT_PUBLIC_KEY
+        || pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY )
         pk = pkt->pkt.public_key;
-    else if (pkt->pkttype == CDK_PKT_SECRET_KEY
-             || pkt->pkttype == CDK_PKT_SECRET_SUBKEY)
+    else if( pkt->pkttype == CDK_PKT_SECRET_KEY
+             || pkt->pkttype == CDK_PKT_SECRET_SUBKEY ) {
         pk = pkt->pkt.secret_key->pk;
+        is_sk = 1;
+    }
     else
         return 0;
-    return !pk->is_revoked && !pk->has_expired;
+    valid = !pk->is_revoked && !pk->has_expired;
+    if( is_sk )
+        return valid;
+    return valid && !pk->is_invalid;
 }
 
 
 static CDK_KBNODE
-keydb_find_byusage (CDK_KBNODE root, int req_usage, int pk)
+keydb_find_byusage( CDK_KBNODE root, int req_usage, int pk )
 {
     CDK_KBNODE node;
-    int pkttype = 0;
+    int pkttype = 0, req_type = 0;
 
-    if (!req_usage)
-        return cdk_kbnode_find (root, CDK_PKT_PUBLIC_KEY);
+    req_type = pk? CDK_PKT_PUBLIC_KEY : CDK_PKT_SECRET_KEY;
+    if( !req_usage )
+        return cdk_kbnode_find( root, req_type );
 
-    for (node = root; node; node = node->next) {
+    node = cdk_kbnode_find( root, req_type );
+    if( node && !keydb_check_key( node->pkt ) )
+        return NULL;
+
+    for( node = root; node; node = node->next ) {
         pkttype = node->pkt->pkttype;
-        if (pk && (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
+        if( pk && (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
                    || node->pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
-            && keydb_check_key (node->pkt)
-            && (node->pkt->pkt.public_key->pubkey_usage & req_usage))
+            && keydb_check_key( node->pkt )
+            && (node->pkt->pkt.public_key->pubkey_usage & req_usage) )
             return node;
-        if (!pk && (node->pkt->pkttype == CDK_PKT_SECRET_KEY
+        if( !pk && (node->pkt->pkttype == CDK_PKT_SECRET_KEY
                     || node->pkt->pkttype == CDK_PKT_SECRET_SUBKEY)
-            && keydb_check_key (node->pkt)
-            && (node->pkt->pkt.secret_key->req_usage & req_usage))
+            && keydb_check_key( node->pkt )
+            && (node->pkt->pkt.secret_key->pk->pubkey_usage & req_usage) )
             return node;
     }
     return NULL;
@@ -824,90 +835,74 @@ keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid)
 
 
 int
-_cdk_keydb_get_sk_byname (CDK_KEYDB_HD hd, const char * name,
-                          cdkPKT_secret_key ** ret_sk, int alloced)
+_cdk_keydb_get_sk_byusage( CDK_KEYDB_HD hd, const char * name,
+                           cdkPKT_secret_key ** ret_sk, int usage )
 {
     CDK_KBNODE knode = NULL, node = NULL;
     cdkPKT_secret_key * sk = NULL;
-    int req_usage = 0;
     int rc = 0;
 
-    if (!ret_sk)
+    if( !ret_sk || !usage )
         return CDK_Inv_Value;
-    if (!hd)
+    if( !hd )
         return CDK_Error_No_Keyring;
   
-    rc = cdk_keydb_search_auto (hd, name, &knode);
-    if (rc)
+    rc = cdk_keydb_search_auto( hd, name, &knode );
+    if( rc )
         goto leave;
-    if (alloced) {
-        req_usage = ret_sk[0]->req_usage;
-        cdk_free (ret_sk[0]);
-        node = keydb_find_byusage (knode, req_usage, 0);
-    }
-    if (!node)
-        node = cdk_kbnode_find (knode, CDK_PKT_SECRET_KEY);
-    if (!node) {
-        rc = CDK_Error_No_Key;
+    node = keydb_find_byusage( knode, usage, 0 );
+    if( !node ) {
+        rc = CDK_Unusable_Key;
         goto leave;
     }
-    sk = node->pkt->pkt.secret_key;
-    _cdk_kbnode_clone (node);
-    cdk_kbnode_release (knode);
 
- leave:
+    sk = node->pkt->pkt.secret_key;
+    _cdk_kbnode_clone( node );
+    cdk_kbnode_release( knode );
+
+leave:
     *ret_sk = sk;
     return rc;
 }
 
 
 int
-_cdk_keydb_get_pk_byname (CDK_KEYDB_HD hd, const char * name,
-                          cdkPKT_public_key ** ret_pk, int alloced)
+_cdk_keydb_get_pk_byusage( CDK_KEYDB_HD hd, const char * name,
+                           cdkPKT_public_key ** ret_pk, int usage )
 {
     CDK_KBNODE knode, node = NULL;
     cdkPKT_public_key * pk = NULL;
     const char * s;
-    int req_usage = 0;
     int rc = 0;
 
-    if (!ret_pk)
+    if( !ret_pk || !usage )
         return CDK_Inv_Value;
-    if (!hd)
+    if( !hd )
         return CDK_Error_No_Keyring;
   
-    rc = cdk_keydb_search_auto (hd, name, &knode);
-    if (rc)
-        goto leave;
-    if (alloced) {
-        req_usage = ret_pk[0]->pubkey_usage;
-        cdk_free (ret_pk[0]);
-        node = keydb_find_byusage (knode, req_usage, 1);
-    }
-    if (!node)
-        node = cdk_kbnode_find (knode, CDK_PKT_PUBLIC_KEY);
-    if (!node) {
-        rc = CDK_Error_No_Key;
+    rc = cdk_keydb_search_auto( hd, name, &knode );
+    if( rc )
+        goto leave;    
+    node = keydb_find_byusage( knode, usage, 1 );
+    if( !node ) {
+        rc = CDK_Unusable_Key;
         goto leave;
     }
 
-    _cdk_copy_pubkey (&pk, node->pkt->pkt.public_key);
-    for (node = knode; node; node = node->next) {
-        if (node->pkt->pkttype == CDK_PKT_USER_ID) {
+    _cdk_copy_pubkey( &pk, node->pkt->pkt.public_key );
+    for( node = knode; node; node = node->next ) {
+        if( node->pkt->pkttype == CDK_PKT_USER_ID ) {
             s = node->pkt->pkt.user_id->name;
-            if (pk && !pk->uid && _cdk_memistr (s, strlen (s), name)) {
-                _cdk_copy_userid (&pk->uid, node->pkt->pkt.user_id);
+            if( pk && !pk->uid && _cdk_memistr( s, strlen( s ), name ) ) {
+                _cdk_copy_userid( &pk->uid, node->pkt->pkt.user_id );
                 break;
 	    }
 	}
     }
-    /*_cdk_log_debug ("chose `%s' as the user id\n",
-      pk->uid? pk->uid->name : "[null]");*/
-    cdk_kbnode_release (knode);
+    cdk_kbnode_release( knode );
 
- leave:
-    if (ret_pk)
-        *ret_pk = pk;
+leave:
+    *ret_pk = pk;
     return rc;
 }
 
@@ -1108,7 +1103,8 @@ keydb_parse_allsigs( CDK_KBNODE knode, int check )
     cdkPKT_signature * sig;
     cdkPKT_public_key * pk;
     struct cdk_subpkt_s * s = NULL;
-    u32 expiredate = 0, curtime = _cdk_timestamp (), keyid[2];
+    u32 expiredate = 0, curtime = _cdk_timestamp ();
+    u32 keyid[2];
     int rc = 0;
 
     if( !knode )
@@ -1151,7 +1147,7 @@ keydb_parse_allsigs( CDK_KBNODE knode, int check )
                 kb = cdk_kbnode_find_prev (knode, node, CDK_PKT_PUBLIC_SUBKEY);
                 if( kb ) {
                     kb->pkt->pkt.public_key->is_revoked = 1;
-                    if (check)
+                    if( check )
                         ;
 		}
                 else
@@ -1178,12 +1174,12 @@ keydb_parse_allsigs( CDK_KBNODE knode, int check )
                     pk->is_invalid = 0;
                     s = cdk_subpkt_find( node->pkt->pkt.signature->hashed,
                                          SIGSUBPKT_KEY_EXPIRE );
-                    if ( s ) {
-                        expiredate = _cdk_buftou32 (s->d);
+                    if( s ) {
+                        expiredate = _cdk_buftou32( s->d );
                         pk->expiredate = pk->timestamp + expiredate;
                         pk->has_expired = pk->expiredate > curtime? 0 : 1;
 		    }
-                    if (check)
+                    if( check )
                         ;
 		}
                 else
@@ -1197,12 +1193,12 @@ keydb_parse_allsigs( CDK_KBNODE knode, int check )
                     pk->is_invalid = 0;
                     s = cdk_subpkt_find( node->pkt->pkt.signature->hashed,
                                          SIGSUBPKT_KEY_EXPIRE );
-                    if (s) {
-                        expiredate = _cdk_buftou32 (s->d);
+                    if( s ) {
+                        expiredate = _cdk_buftou32( s->d );
                         pk->expiredate = pk->timestamp + expiredate;
                         pk->has_expired = pk->expiredate > curtime? 0 : 1;
 		    }
-                    if (check)
+                    if( check )
                         ;
 		}
                 else
@@ -1222,6 +1218,15 @@ keydb_parse_allsigs( CDK_KBNODE knode, int check )
             }
         }
     }
+    if( node && (node->pkt->pkt.public_key->is_revoked
+                 || node->pkt->pkt.public_key->has_expired) ) {
+        /* if the primary key has been revoked, mark all subkeys as invalid
+           because without a primary key they are not useable */
+        for( node = knode; node; node = node->next ) {
+            if( node->pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY )
+                node->pkt->pkt.public_key->is_invalid = 1;
+        }
+    }
     return rc;
 }
 
@@ -1292,8 +1297,8 @@ cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * r_knode)
     }
 
     if( got_key ) {
-        keydb_merge_selfsig (knode, main_keyid);
-        rc = keydb_parse_allsigs (knode, 0);
+        keydb_merge_selfsig( knode, main_keyid );
+        rc = keydb_parse_allsigs( knode, 0 );
     }
     if( r_knode )
         *r_knode = got_key ? knode : NULL;
@@ -1424,7 +1429,7 @@ cdk_keydb_export (CDK_KEYDB_HD hd, CDK_STREAM out, CDK_STRLIST remusr)
                 !node->pkt->pkt.signature->flags.exportable)
                 continue;
             if (node->pkt->pkttype == CDK_PKT_SIGNATURE
-                && node->pkt->pkt.signature->pubkey_algo == 16)
+                && !KEY_CAN_SIGN (node->pkt->pkt.signature->pubkey_algo))
                 continue;
             if (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
                 && node->pkt->pkt.public_key->version == 3)
