@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "bsd" -*- */
 /* main.c
- *       Copyright (C) 2001, 2002 Timo Schulz 
+ *       Copyright (C) 2001, 2002, 2003 Timo Schulz 
  *
  * This file is part of OpenCDK.
  *
@@ -34,6 +34,7 @@
 
 #include "opencdk.h"
 #include "main.h"
+#include "packet.h"
 
 static void *(*alloc_func) (size_t n) = gcry_xmalloc;
 static void *(*alloc_secure_func) (size_t n) = gcry_malloc_secure;
@@ -51,9 +52,10 @@ static CDK_PASSPHRASE_CB passphrase_cb = NULL;
 
 
 /**
- * cdk_strerror: Return an error text for the given id
+ * cdk_strerror:
  * @ec: the error number
  *
+ * Return an error text for the given id.
  **/
 const char *
 cdk_strerror (int ec)
@@ -86,6 +88,7 @@ cdk_strerror (int ec)
     case CDK_Error_No_Keyring: return "No keyring available";
     case CDK_Inv_Packet_Ver:   return "Invalid version for packet";
     case CDK_Too_Short:        return "Buffer or object is too short";
+    case CDK_Unusable_Key:     return "Unusable public key";
     default:                   sprintf (buf, "ec=%d", ec); return buf;
     }
     return NULL;
@@ -100,13 +103,14 @@ out_of_core (size_t n)
 
 
 /**
- * cdk_set_malloc_hooks: Set private memory hooks for the lib
+ * cdk_set_malloc_hooks: 
  * @new_alloc_func: malloc replacement
  * @new_alloc_secure_func: secure malloc replacement
  * @new_realloc_func: realloc replacement
  * @new_calloc_func: calloc replacement
  * @new_free_func: free replacement
  *
+ * Set private memory hooks for the lib.
  */
 void
 cdk_set_malloc_hooks (void *(*new_alloc_func) (size_t n),
@@ -124,6 +128,11 @@ cdk_set_malloc_hooks (void *(*new_alloc_func) (size_t n),
 }
 
 
+/**
+ * cdk_malloc_hook_initialized:
+ *
+ * Return if the malloc hooks are already initialized.
+ **/
 int
 cdk_malloc_hook_initialized (void)
 {
@@ -247,10 +256,11 @@ _cdk_logv (int level, const char *fmt, va_list arg_ptr)
 
 
 /**
- * cdk_set_log_handler: set a private handler for logging.
+ * cdk_set_log_handler: 
  * @logfnc: the function pointer
  * @opaque: a private values for the function
  *
+ * set a private handler for logging.
  */
 void
 cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque)
@@ -261,9 +271,10 @@ cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque)
 
 
 /**
- * cdk_set_log_level: set the verbosity level
+ * cdk_set_log_level: 
  * @lvl: the level
  *
+ * set the verbosity level.
  **/
 void
 cdk_set_log_level (int lvl)
@@ -323,24 +334,23 @@ getpass (const char * prompt)
 
 
 char *
-_cdk_passphrase_get (const char * prompt)
+_cdk_passphrase_get( const char * prompt )
 {
-    if (passphrase_cb)
-        return passphrase_cb (prompt);
-    return getpass (prompt);
+    char * p, * pass = NULL;
+    
+    if( passphrase_cb )
+        return passphrase_cb( prompt );
+    p = getpass( prompt );
+    if( p )
+        pass = cdk_strdup( p );
+    return pass;
 }
 
 
 void
-_cdk_passphrase_free (char *pw)
+_cdk_passphrase_free( char *pw )
 {
-    if (passphrase_cb) {
-        cdk_free (pw);
-        return;
-    }
-    /* if we use getpass, we've to free the handle with free() because
-       it was *not* allocated with the gcry memory routines */
-    free (pw);
+    cdk_free( pw );
 }
 
 
@@ -492,6 +502,18 @@ cdk_handle_control( CDK_HD hd, int action, int cmd, ... )
         else
             val = hd->_s2k.mode;
         break;
+
+    case CDK_CTL_KEYCACHE_ON:
+        if( set )
+            hd->cache.on = va_arg( arg_ptr, int );
+        else
+            val = hd->cache.on;
+        break;
+
+    case CDK_CTL_KEYCACHE_FREE:
+        _cdk_free_seckey( hd->cache.sk );
+        hd->cache.sk = NULL;
+        break;
     }
     va_end( arg_ptr );
     return val;
@@ -500,9 +522,10 @@ cdk_handle_control( CDK_HD hd, int action, int cmd, ... )
             
 
 /**
- * cdk_handle_new: create a new session handle
+ * cdk_handle_new:
  * @r_ctx: context to store the handle
  *
+ * create a new session handle.
  **/
 int
 cdk_handle_new (CDK_HD * r_ctx)
@@ -536,10 +559,11 @@ cdk_handle_new (CDK_HD * r_ctx)
 
 
 /**
- * cdk_handle_set_keydb: set the key database handle
+ * cdk_handle_set_keydb:
  * @hd: session handle
  * @db: the database handle
  *
+ * set the key database handle.
  * the function automatically detects whether this is a public or
  * secret keyring and the right handle is set.
  **/
@@ -569,11 +593,12 @@ cdk_handle_get_keydb( CDK_HD hd, int type )
 
 
 /**
- * cdk_handle_set_callback: set the callback for filter operations
+ * cdk_handle_set_callback: 
  * @hd: the handle
  * @cb: the callback function
  * @cb_value: the opaque value for the function
  *
+ * set the callback for filter operations.
  **/
 void
 cdk_handle_set_callback (CDK_HD hd,
@@ -588,18 +613,20 @@ cdk_handle_set_callback (CDK_HD hd,
 
 
 /**
- * cdk_handle_free: free the main handle
+ * cdk_handle_free:
  * @hd: the handle
  *
+ * free the main handle.
  **/
 void
-cdk_handle_free (CDK_HD hd)
+cdk_handle_free( CDK_HD hd )
 {
     if( !hd )
         return;
-    _cdk_result_verify_free (hd->result.verify);
-    cdk_free (hd->s2k);
-    cdk_free (hd->dek);
-    cdk_free (hd);
+    _cdk_result_verify_free( hd->result.verify );
+    _cdk_free_seckey( hd->cache.sk );
+    cdk_free( hd->s2k );
+    cdk_free( hd->dek );
+    cdk_free( hd );
 }
 
