@@ -68,7 +68,8 @@ int _cdk_strcmp (const char * a, const char * b);
 u32 _cdk_buftou32 (const byte * buf);
 void _cdk_u32tobuf (u32 u, byte * buf);
 const char * _cdk_memistr (const char * buf, size_t buflen, const char * sub);
-
+void _cdk_vasprintf_free (void * p);
+    
 /*-- pubkey.c --*/
 u32 _cdk_pkt_get_keyid( CDK_PACKET * pkt, u32 * keyid );
 int _cdk_fingerprint_get (CDK_PACKET * pkt, byte * fpr);
@@ -105,10 +106,10 @@ int _cdk_digest_encode_pkcs1( byte ** r_md, size_t * r_mdlen, int pk_algo,
 int _cdk_sk_unprotect_auto (cdkPKT_secret_key * sk);
 
 /*-- keydb.c --*/
-int _cdk_keydb_get_pk_byname (CDK_KEYDB_HD hd, const char * name,
-                              cdkPKT_public_key ** ret_pk, int alloced);
-int _cdk_keydb_get_sk_byname (CDK_KEYDB_HD hd, const char * name,
-                              cdkPKT_secret_key ** ret_sk, int alloced);
+int _cdk_keydb_get_pk_byusage (CDK_KEYDB_HD hd, const char * name,
+                              cdkPKT_public_key ** ret_pk, int usage);
+int _cdk_keydb_get_sk_byusage (CDK_KEYDB_HD hd, const char * name,
+                              cdkPKT_secret_key ** ret_sk, int usage);
 
 /*-- sign.c --*/
 int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig);
@@ -134,7 +135,7 @@ Verify_Result _cdk_result_verify_new (void);
 /*-- encrypt.c --*/
 int _cdk_check_args( int overwrite, const char * in, const char * out );
 int _cdk_proc_packets( CDK_HD hd, CDK_STREAM inp, const char * output,
-                       GCRY_MD_HD md );
+                       CDK_STREAM outstream, GCRY_MD_HD md );
 
 /*-- read-packet.c --*/
 size_t _cdk_pkt_read_len (FILE * inp, int * ret_partial);
