@@ -1,5 +1,5 @@
 /* opencdk.h - Open Crypto Development Kit (OpenCDK)
- *        Copyright (C) 2001, 2002 Timo Schulz
+ *        Copyright (C) 2001, 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -21,10 +21,14 @@
 #ifndef OPENCDK_H
 #define OPENCDK_H
 
-#define OPENCDK_VERSION "0.4.0"
+#include <stdarg.h>
 
-#include <gcrypt.h>
+#define OPENCDK_VERSION "0.4.2"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+    
 /* General contexts */
 struct cdk_handle_s;
 typedef struct cdk_handle_s *CDK_HD;
@@ -32,6 +36,9 @@ typedef struct cdk_handle_s *CDK_HD;
 struct cdk_string_list_s;
 typedef struct cdk_string_list_s *CDK_STRLIST;
 
+struct cdk_sesskey_s;
+typedef struct cdk_sesskey_s *CDK_SESSKEY;
+    
 struct cdk_mpi_s;
 typedef struct cdk_mpi_s *CDK_MPI;
 
@@ -93,9 +100,26 @@ typedef enum
   CDK_Error_No_Keyring = 21,
   CDK_Wrong_Format = 22,
   CDK_Inv_Packet_Ver = 23,
-  CDK_Too_Short = 24
+  CDK_Too_Short = 24,
+  CDK_Unusable_Key = 25,
 } cdk_error_t;
 
+
+enum
+{
+  CDK_CTLF_SET          =  0,
+  CDK_CTLF_GET          =  1,
+  CDK_CTL_DIGEST        = 10,
+  CDK_CTL_CIPHER        = 11,
+  CDK_CTL_ARMOR         = 12,
+  CDK_CTL_COMPRESS      = 13,
+  CDK_CTL_COMPAT        = 14,
+  CDK_CTL_OVERWRITE     = 15,
+  CDK_CTL_S2K           = 16,
+  CDK_CTL_KEYCACHE_ON   = 17,
+  CDK_CTL_KEYCACHE_FREE = 18
+};
+
 enum
 {
   CDK_LOG_NONE  = 0,
@@ -122,11 +146,28 @@ enum
 
 enum
 {
-  CDK_MD_MD5    = 1,
-  CDK_MD_SHA1   = 2,
-  CDK_MD_RMD160 = 3
+  CDK_MD_NONE    = 0,
+  CDK_MD_MD5     = 1,
+  CDK_MD_SHA1    = 2,
+  CDK_MD_RMD160  = 3,
+  CDK_MD_MD2     = 5,
+  CDK_MD_TIGER   = 6,
 };
 
+enum
+{
+  CDK_CIPHER_NONE        = 0,
+  CDK_CIPHER_IDEA        = 1,
+  CDK_CIPHER_3DES        = 2,
+  CDK_CIPHER_CAST5       = 3,
+  CDK_CIPHER_BLOWFISH    = 4,
+  CDK_CIPHER_SAFER_SK128 = 5,
+  CDK_CIPHER_DES_SK      = 6,
+  CDK_CIPHER_AES         = 7,
+  CDK_CIPHER_AES192      = 8,
+  CDK_CIPHER_AES256      = 9,
+  CDK_CIPHER_TWOFISH     = 10,
+};
 
 enum
 {
@@ -147,7 +188,8 @@ enum
 
 enum
 {
-  CDK_STREAMCTL_RESET = 1
+  CDK_STREAMCTL_RESET   = 1,
+  CDK_STREAMCTL_DISABLE = 2,
 };
 
 enum
@@ -166,6 +208,17 @@ enum
 #define CDK_DBTYPE_ARMORED 100
 #define CDK_DBTYPE_KEYRING 100
 
+
+enum
+{
+  CDK_CRYPTYPE_NONE    = 0,
+  CDK_CRYPTYPE_ENCRYPT = 1,
+  CDK_CRYPTYPE_DECRYPT = 2,
+  CDK_CRYPTYPE_SIGN    = 3,
+  CDK_CRYPTYPE_VERIFY  = 4,
+  CDK_CRYPTYPE_EXPORT  = 5,
+};
+
 enum
 {
   CDK_KEY_VALID   = 0,
@@ -188,23 +241,38 @@ enum
   CDK_TFLAG_DISABLED    = 128
 };
 
+
 enum
 {
+  /* signature status */
   CDK_SIGSTAT_NONE  = 0,
   CDK_SIGSTAT_GOOD  = 1,
   CDK_SIGSTAT_BAD   = 2,
-  CDK_SIGSTAT_NOKEY = 3
+  CDK_SIGSTAT_NOKEY = 3,
+  /* signature modes */
+  CDK_SIGMODE_NORMAL   = 100,
+  CDK_SIGMODE_DETACHED = 101,
+  CDK_SIGMODE_CLEAR    = 102
 };
 
 enum
 {
+  /* cdk attributes */
   CDK_ATTR_CREATED  = 1,
   CDK_ATTR_EXPIRE   = 2,
   CDK_ATTR_KEYID    = 3,
   CDK_ATTR_STATUS   = 4,
   CDK_ATTR_NOTATION = 5,
   CDK_ATTR_ALGO_PK  = 6,
-  CDK_ATTR_ALGO_MD  = 7
+  CDK_ATTR_ALGO_MD  = 7,
+  CDK_ATTR_VERSION  = 8,
+  CDK_ATTR_LEN      = 9,
+  CDK_ATTR_FLAGS    = 10,
+  CDK_ATTR_MPI      = 11,
+  /* cdk key flags */
+  CDK_FLAG_KEY_REVOKED = 256,
+  CDK_FLAG_KEY_EXPIRED = 512,
+  CDK_FLAG_SIG_EXPIRED = 1024,  
 };
 
 
@@ -238,6 +306,8 @@ typedef enum
   CDK_PKT_MDC           = 19,
 } cdk_packet_types_t;
 
+#define CDK_PKT_IS_ENCRYPTED(pkttype) \
+(((pkttype) == CDK_PKT_ENCRYPTED_MDC) || ((pkttype) == CDK_PKT_ENCRYPTED))
 
 typedef struct
 {
@@ -298,7 +368,6 @@ typedef struct
   unsigned is_primary:1;
   unsigned has_expired:1;
   unsigned is_revoked:1;
-  int req_usage;
 } cdkPKT_secret_key;
 
 
@@ -311,7 +380,7 @@ typedef struct
   unsigned long keyid[2];
   unsigned char pubkey_algo;
   unsigned char digest_algo;
-  unsigned char digest_start[2];  
+  unsigned char digest_start[2];
   unsigned short hashed_size;
   CDK_SUBPKT hashed;
   unsigned short unhashed_size;
@@ -422,6 +491,7 @@ typedef struct cdk_packet_s CDK_PACKET;
 
 
 /*-- main.c --*/
+/* memory routines */
 typedef void (*CDK_LOG_FNC) (void *, int, const char *, va_list);
 void cdk_set_log_level (int lvl);
 void cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque);
@@ -435,42 +505,48 @@ int cdk_malloc_hook_initialized (void);
 void * cdk_malloc (size_t size);
 void * cdk_calloc (size_t n, size_t m);
 void * cdk_realloc (void * ptr, size_t size);
-void * cdk_salloc (size_t size);
-void * cdk_scalloc (size_t m);
+void * cdk_salloc (size_t size, int clear);
 char * cdk_strdup (const char * ptr);
 void cdk_free (void * ptr);
-
+/* session handle routines */
 int cdk_handle_new (CDK_HD * r_ctx);
 void cdk_handle_free (CDK_HD c);
 void cdk_handle_set_keydb (CDK_HD hd, CDK_KEYDB_HD db);
-void cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher);
-void cdk_handle_set_cipher (CDK_HD hd, int cipher);
-void cdk_handle_set_digest (CDK_HD hd, int digest);
-void cdk_handle_set_compress (CDK_HD hd, int algo);
-void cdk_handle_set_armor (CDK_HD hd, int val);
-int cdk_handle_get_armor (CDK_HD hd);
-void cdk_handle_set_compat (CDK_HD hd, int val);
+CDK_KEYDB_HD cdk_handle_get_keydb( CDK_HD hd, int type );
+int cdk_handle_control( CDK_HD hd, int action, int cmd, ... );
 void cdk_handle_set_callback (CDK_HD hd,
                               void (*cb) (void *opa, int type, const char * s),
                               void * cb_value);
 
+/* shortcuts for some controls */
+#define cdk_handle_set_armor( a, val ) \
+  cdk_handle_control( (a), CDK_CTLF_SET, CDK_CTL_ARMOR, (val) )
+
+#define cdk_handle_set_compress( a, algo, level ) \
+  cdk_handle_control( (a), CDK_CTLF_SET, CDK_CTL_COMPRESS, (algo), (level) )
+
+
 /*-- cipher.c --*/
 void cdk_set_progress_handler (void (*cb)(void * hd, unsigned off,
                                           unsigned size), void * cb_value);
 
 /*-- new-packet.c --*/
 int cdk_pkt_new (CDK_PACKET ** r_pkt);
+void cdk_pkt_init( CDK_PACKET * pkt );
 int cdk_pkt_alloc (CDK_PACKET ** r_pkt, int id);
 void cdk_pkt_free (CDK_PACKET * pkt);
 void cdk_pkt_release (CDK_PACKET * pkt);
 int cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt);
 int cdk_pkt_build (CDK_STREAM out, CDK_PACKET * pkt);
+/* sub packet routines */
 CDK_SUBPKT cdk_subpkt_new (size_t size);
 void cdk_subpkt_free (CDK_SUBPKT ctx);
 CDK_SUBPKT cdk_subpkt_find (CDK_SUBPKT ctx, int type);
 const unsigned char * cdk_subpkt_get_buf (CDK_SUBPKT ctx,
                                           int * r_type,
                                           size_t * r_nbytes);
+unsigned char  * cdk_prefs_get_array( cdkPKT_user_id * id, int type,
+                                      size_t *ret_len );
 
 /*-- pubkey.c --*/
 #define is_RSA(a) ((a) == CDK_PK_RSA \
@@ -480,9 +556,9 @@ const unsigned char * cdk_subpkt_get_buf (CDK_SUBPKT ctx,
 #define is_DSA(a) ((a) == CDK_PK_DSA)
 
 int cdk_pk_encrypt (cdkPKT_public_key * pk, cdkPKT_pubkey_enc * pke,
-                    GCRY_MPI esk);
+                    CDK_SESSKEY esk);
 int cdk_pk_decrypt (cdkPKT_secret_key * sk, cdkPKT_pubkey_enc * pke,
-                    GCRY_MPI * r_sk);
+                    CDK_SESSKEY *r_sk);
 int cdk_pk_sign (cdkPKT_secret_key * sk, cdkPKT_signature * sig,
                  const unsigned char * md);
 int cdk_pk_verify (cdkPKT_public_key * pk, cdkPKT_signature * sig,
@@ -504,20 +580,28 @@ unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long * keyid);
 unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long * keyid);
 unsigned long cdk_sig_get_keyid (cdkPKT_signature * sg, unsigned long * keyid);
 typedef char * (*CDK_PASSPHRASE_CB) (const char *);
-int cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw);
-CDK_DEK cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD s2k,
-                               int mode, char * pw);
+int cdk_sk_unprotect (cdkPKT_secret_key * sk, const char * pw);
+int cdk_sk_protect (cdkPKT_secret_key * sk, const char * pw);
 void cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc);
 
-/*-- verify.c --*/
-unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
-const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
-     
 /*-- seskey.c --*/
-CDK_DEK cdk_pkcs1_to_dek (cdkPKT_pubkey_enc * enc, cdkPKT_secret_key * sk);
-int cdk_dek_new (CDK_DEK * r_dek, int algo);
+int cdk_sesskey_new( CDK_SESSKEY * r_sk );
+void cdk_sesskey_free( CDK_SESSKEY sk );
+int cdk_dek_new( CDK_DEK * r_dek );
+void cdk_dek_free( CDK_DEK dek );
+int cdk_dek_set_cipher( CDK_DEK dek, int algo );
+int cdk_dek_set_key( CDK_DEK dek, const unsigned char *key, size_t keylen );
+int cdk_dek_from_passphrase( CDK_DEK * ret_dek, int cipher_algo,
+                             CDK_S2K_HD s2k, int mode, const char * pw );
+int cdk_dek_encode_pkcs1( CDK_DEK dek, int nbits, CDK_SESSKEY * r_esk );
+int cdk_dek_decode_pkcs1( CDK_DEK * ret_dek, CDK_SESSKEY esk );
+int cdk_dek_extract( CDK_DEK * ret_dek, cdkPKT_pubkey_enc * enc,
+                     cdkPKT_secret_key * sk );
+void cdk_dek_set_mdc_flag( CDK_DEK dek, int val );
+/* string to key */
 int cdk_s2k_new (CDK_S2K_HD * ret_s2k, int mode, int algo,
                  const unsigned char * salt);
+void cdk_s2k_free (CDK_S2K_HD s2k);
 
 /*-- armor.c --*/
 int cdk_file_armor (CDK_HD hd, const char * file, const char * output);
@@ -528,6 +612,7 @@ int cdk_armor_filter_use (CDK_STREAM inp);
 int cdk_stream_control (CDK_STREAM s, int ctl, int val);
 int cdk_stream_open (const char * file, CDK_STREAM * ret_s);
 int cdk_stream_new (const char * file, CDK_STREAM * ret_s);
+int cdk_stream_create (const char * file, CDK_STREAM * ret_s);
 CDK_STREAM cdk_stream_tmp (void);
 CDK_STREAM cdk_stream_tmp_from_mem (const void * buf, size_t count);
 void cdk_stream_tmp_set_mode (CDK_STREAM s, int val);
@@ -542,12 +627,12 @@ int cdk_stream_write (CDK_STREAM s, const void * buf, size_t count);
 int cdk_stream_putc (CDK_STREAM s, int c);
 int cdk_stream_getc (CDK_STREAM s);
 int cdk_stream_eof (CDK_STREAM s);
-int cdk_stream_tell (CDK_STREAM s);
+long cdk_stream_tell (CDK_STREAM s);
 int cdk_stream_seek (CDK_STREAM s, long offset);
 int cdk_stream_set_armor_flag (CDK_STREAM s, int type);
 int cdk_stream_set_literal_flag (CDK_STREAM s, int mode, const char * fname);
 int cdk_stream_set_cipher_flag (CDK_STREAM s, CDK_DEK dek, int use_mdc);
-int cdk_stream_set_compress_flag (CDK_STREAM s, int algo);
+int cdk_stream_set_compress_flag (CDK_STREAM s, int algo, int level);
 int cdk_stream_set_hash_flag (CDK_STREAM s, int algo);
 int cdk_stream_set_text_flag (CDK_STREAM s, const char * lf);
 int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
@@ -555,12 +640,12 @@ int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
 /*-- keydb.c --*/
 int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count);
 int cdk_keydb_open (CDK_KEYDB_HD hd, CDK_STREAM * ret_kr);
+int cdk_keydb_check_sk( CDK_KEYDB_HD hd, unsigned long * keyid );
 int cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc);
 void cdk_keydb_search_free (CDK_DBSEARCH dbs);
 int cdk_keydb_search (CDK_KEYDB_HD hd, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
 int cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc,
                            CDK_KBNODE * r_knode);
-void cdk_keydb_search_reset (CDK_KEYDB_HD hd);
 void cdk_keydb_free (CDK_KEYDB_HD hd);
 int cdk_keydb_get_bykeyid (CDK_KEYDB_HD hd, unsigned long * keyid,
                            CDK_KBNODE * r_key);
@@ -579,13 +664,11 @@ int cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4]);
 /*-- kbnode.c --*/
 CDK_KBNODE cdk_kbnode_new (CDK_PACKET * pkt);
 int cdk_kbnode_read_from_mem (CDK_KBNODE * ret_node,
-                             const unsigned char * buf,
-                             size_t buflen);
+                             const unsigned char * buf, size_t buflen);
 int cdk_kbnode_write_to_mem (CDK_KBNODE node,
-                             unsigned char * buf,
-                             size_t * r_nbytes);
+                             unsigned char * buf, size_t * r_nbytes);
 void cdk_kbnode_release (CDK_KBNODE n);
-CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * context, int all);
+CDK_KBNODE cdk_kbnode_walk (CDK_KBNODE root, CDK_KBNODE * ctx, int all);
 CDK_PACKET * cdk_kbnode_find_packet (CDK_KBNODE node, int pkttype);
 CDK_PACKET * cdk_kbnode_get_packet (CDK_KBNODE node);
 CDK_KBNODE cdk_kbnode_find (CDK_KBNODE node, int pkttype);
@@ -593,7 +676,8 @@ CDK_KBNODE cdk_kbnode_find_prev (CDK_KBNODE root, CDK_KBNODE node, int pkttyp);
 CDK_KBNODE cdk_kbnode_find_next (CDK_KBNODE node, int pkttype);
 
 /*-- sig-check.c --*/
-int cdk_key_check_sigs (CDK_KBNODE knode, CDK_KEYDB_HD hd, int * r_status);
+int cdk_pk_check_sigs (CDK_KBNODE knode, CDK_KEYDB_HD hd, int * r_status);
+#define cdk_key_check_sigs cdk_pk_check_sigs
 
 /*-- keylist.c --*/
 int cdk_pklist_select_algo (CDK_KEYLIST pkl);
@@ -602,32 +686,40 @@ int cdk_pklist_build (CDK_KEYDB_HD hd, CDK_STRLIST remusr,
                       CDK_KEYLIST * ret_pkl, int use);
 void cdk_pklist_release (CDK_KEYLIST pkl);
 int cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM out);
+/* secret key list */
 int cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
                       CDK_KEYLIST * ret_skl, int unlock, unsigned int use);
 void cdk_sklist_release (CDK_KEYLIST skl);
-int cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
+int cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, void * mdctx,
                       int sigclass, int sigver);
 int cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass);
 
 /*-- encrypt.c --*/
 int cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
-                        CDK_STREAM inp,
-                        CDK_STREAM out);
+                        CDK_STREAM inp, CDK_STREAM out);
 int cdk_file_encrypt (CDK_HD hd, CDK_STRLIST remusr,
-                      const char * file,
-                      const char * output);
-int cdk_file_decrypt (CDK_HD hd, const char * file,
-                      const char * output);
-
-/*-- verify.c --*/
-int cdk_file_verify (CDK_HD hd, const char * file, const char * output);
+                      const char * file, const char * output);
+int cdk_file_decrypt (CDK_HD hd, const char * file, const char * output);
+int cdk_data_transform( CDK_HD hd, int mode,
+                        CDK_STRLIST locusr, CDK_STRLIST remusr,
+                        const void * inbuf, size_t insize,
+                        unsigned char ** outbuf, size_t * outsize,
+                        int modval );
 
 /*-- sign.c --*/
+int cdk_stream_sign( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out,
+                     CDK_STRLIST locusr, CDK_STRLIST remusr,
+                     int encryptflag, int sigmode );
 int cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
                    const char * file, const char * output,
-                   int detached, int encryptflag);
-int cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
-                        const char * file, const char * output);
+                   int sigmode, int encryptflag);
+
+/*-- verify.c --*/
+int cdk_stream_verify( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out );
+int cdk_file_verify (CDK_HD hd, const char * file, const char * output);
+unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
+const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
+unsigned char * cdk_sig_get_info_as_xml( CDK_HD hd, int advanced );
 
 /*-- trustdb.c --*/
 int cdk_trustdb_get_validity (CDK_STREAM inp, cdkPKT_user_id * id, int *r_val);
@@ -639,7 +731,6 @@ void cdk_strlist_free (CDK_STRLIST sl);
 CDK_STRLIST cdk_strlist_add (CDK_STRLIST * list, const char * string);
 const char * cdk_strlist_walk (CDK_STRLIST root, CDK_STRLIST * context);
 const char * cdk_check_version (const char * req_version);
-
 /* UTF8 */
 char * cdk_utf8_encode (const char * string);
 char * cdk_utf8_decode (const char * string, size_t length, int delim);
@@ -650,16 +741,21 @@ int cdk_keyserver_recv_key (const char * host, int port,
                             CDK_KBNODE * r_knode);
 
 /*-- keygen.c --*/
-int cdk_keygen_set_prefs (CDK_KEYGEN_CTX hd, int type, unsigned char * array,
-                          size_t n);
+int cdk_keygen_new (CDK_KEYGEN_CTX * r_hd);
+void cdk_keygen_free (CDK_KEYGEN_CTX hd);
+int cdk_keygen_set_prefs (CDK_KEYGEN_CTX hd, int type,
+                          const unsigned char * array, size_t n);
 int cdk_keygen_set_algo_info (CDK_KEYGEN_CTX hd, int type, int algo, int bits);
 void cdk_keygen_set_mdc_feature (CDK_KEYGEN_CTX hd, int val);
 void cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, int type, long timestamp);
 void cdk_keygen_set_name (CDK_KEYGEN_CTX hd, const char * name);
+void cdk_keygen_set_passphrase (CDK_KEYGEN_CTX hd, const char * pass);
 int cdk_keygen_start (CDK_KEYGEN_CTX hd);
 int cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec);
-int cdk_keygen_new (CDK_KEYGEN_CTX * r_hd);
-void cdk_keygen_free (CDK_KEYGEN_CTX hd);
+
+#ifdef __cplusplus
+}
+#endif 
 
 #endif /* OPENCDK_H */
 
