@@ -727,6 +727,7 @@ cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
     }
     if( chksum != sk->csum )
         return CDK_Chksum_Error;
+    sk->is_protected = 0;
     return 0;
 }
 
@@ -898,9 +899,16 @@ cdk_pk_get_keyid( cdkPKT_public_key * pk, u32 * keyid )
 
 
 u32
-cdk_sk_get_keyid( cdkPKT_secret_key * sk, u32 * keyid)
+cdk_sk_get_keyid( cdkPKT_secret_key * sk, u32 * keyid )
 {
-    return sk && sk->pk? cdk_pk_get_keyid( sk->pk, keyid) : 0;
+    u32 kid = 0;
+    
+    if( sk && sk->pk ) {
+        kid = cdk_pk_get_keyid( sk->pk, keyid );
+        sk->keyid[0] = sk->pk->keyid[0];
+        sk->keyid[1] = sk->pk->keyid[1];
+    }
+    return kid;
 }
 
 
