@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "bsd" -*- */
 /* read-packet.c - Read OpenPGP packets
- *        Copyright (C) 2001, 2002 Timo Schulz 
+ *        Copyright (C) 2001, 2002, 2003 Timo Schulz 
  *
  * This file is part of OpenCDK.
  *
@@ -230,8 +230,9 @@ read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
 
     if (partial)
         _cdk_stream_set_blockmode (inp, pktlen);
+    /*enc->len = cdk_stream_get_length (enc->buf) - cdk_stream_tell (inp);*/
+    enc->len = pktlen;
     enc->buf = inp;
-    enc->len = cdk_stream_get_length (enc->buf) - cdk_stream_tell (inp);
     return rc;
 }
 
@@ -289,31 +290,33 @@ read_symkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_symkey_enc * ske)
 
 
 static int
-read_pubkey_enc (CDK_STREAM inp, size_t pktlen, cdkPKT_pubkey_enc * pke)
+read_pubkey_enc( CDK_STREAM inp, size_t pktlen, cdkPKT_pubkey_enc * pke )
 {
     int rc = 0;
     int i, nenc = 0;
 
-    if (!inp || !pke)
+    if( !inp || !pke )
         return CDK_Inv_Value;
 
-    if (DEBUG_PKT)
-        _cdk_log_debug ("** read public key encrypted packet\n");
+    if( DEBUG_PKT )
+        _cdk_log_debug( "** read public key encrypted packet\n" );
 
-    if (pktlen < 10)
+    if( pktlen < 10 )
         return CDK_Inv_Packet;
-    pke->version = stream_getc (inp);
-    if (pke->version < 2 || pke->version > 3)
+    pke->version = stream_getc( inp );
+    if( pke->version < 2 || pke->version > 3 )
         return CDK_Inv_Packet;
-    pke->keyid[0] = read_32 (inp);
-    pke->keyid[1] = read_32 (inp);
-    if (!pke->keyid[0] && !pke->keyid[1])
+    pke->keyid[0] = read_32( inp );
+    pke->keyid[1] = read_32( inp );
+    if( !pke->keyid[0] && !pke->keyid[1] )
         pke->throw_keyid = 1; /* RFC2440 "speculative" keyID */
-    pke->pubkey_algo = stream_getc (inp);
-    nenc = cdk_pk_get_nenc (pke->pubkey_algo);
-    for (i = 0; i < nenc; i++) {
-        rc = read_mpi (inp, &pke->mpi[i], 0);
-        if (rc)
+    pke->pubkey_algo = stream_getc( inp );
+    nenc = cdk_pk_get_nenc( pke->pubkey_algo );
+    if( !nenc )
+        return CDK_Inv_Algo;
+    for( i = 0; i < nenc; i++ ) {
+        rc = read_mpi( inp, &pke->mpi[i], 0 );
+        if( rc )
             break;
     }
     return rc;
@@ -363,37 +366,40 @@ read_compressed (CDK_STREAM inp, size_t pktlen, cdkPKT_compressed * c)
 
 
 static int
-read_public_key (CDK_STREAM inp, cdkPKT_public_key * pk)
+read_public_key( CDK_STREAM inp, cdkPKT_public_key * pk )
 {
-    int i = 0, ndays;
+    int i = 0, ndays, npkey;
     int rc = 0;
 
     if (!inp || !pk)
         return CDK_Inv_Value;
 
-    if (DEBUG_PKT)
-        _cdk_log_debug ("** read public key packet\n");
+    if( DEBUG_PKT )
+        _cdk_log_debug( "** read public key packet\n" );
 
     pk->is_invalid = 1; /* default to detect missing self signatures */
     pk->is_revoked = 0;
     pk->has_expired = 0;
 
-    pk->version = stream_getc (inp);
-    if (pk->version < 2 || pk->version > 4)
+    pk->version = stream_getc( inp );
+    if( pk->version < 2 || pk->version > 4 )
         return CDK_Inv_Packet_Ver;
-    pk->timestamp = read_32 (inp);
-    if (pk->version < 4) {
-        ndays = read_16 (inp);
-        if (ndays)
+    pk->timestamp = read_32( inp );
+    if( pk->version < 4 ) {
+        ndays = read_16( inp );
+        if( ndays ) 
             pk->expiredate = pk->timestamp + ndays * 86400L;
     }
-    pk->pubkey_algo = stream_getc (inp);
-    for (i = 0; i < cdk_pk_get_npkey (pk->pubkey_algo); i++) {
-        rc = read_mpi (inp, &pk->mpi[i], 0);
-        if (rc)
+    pk->pubkey_algo = stream_getc( inp );
+    npkey = cdk_pk_get_npkey( pk->pubkey_algo );
+    if( !npkey )
+        return CDK_Inv_Algo;
+    for( i = 0; i < npkey; i++ ) {
+        rc = read_mpi( inp, &pk->mpi[i], 0 );
+        if ( rc )
             break;
     }
-    pk->pubkey_usage = _cdk_pk_algo_usage (pk->pubkey_algo);
+    pk->pubkey_usage = _cdk_pk_algo_usage( pk->pubkey_algo );
     return rc;
 }
 
@@ -408,7 +414,7 @@ read_public_subkey (CDK_STREAM inp, cdkPKT_public_key * pk)
 
 
 static int
-read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
+read_secret_key( CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk )
 {
     size_t p1 = 0, p2 = 0, nread = 0;
     int i = 0, blklen = 0, nskey = 0;
@@ -447,6 +453,8 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
     if (sk->protect.algo == GCRY_CIPHER_NONE) {
         sk->csum = 0;
         nskey = cdk_pk_get_nskey (sk->pk->pubkey_algo);
+        if( !nskey )
+            return CDK_Inv_Algo;
         for (i = 0; i < nskey; i++) {
             rc = read_mpi (inp, &sk->mpi[i], 1);
             if (rc)
@@ -460,6 +468,8 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
     else if (sk->pk->version < 4) {
         /* mpi size isn't encrypted! */
         nskey = cdk_pk_get_nskey (sk->pk->pubkey_algo);
+        if( !nskey )
+            return CDK_Inv_Algo;
         for (i = 0; i < nskey; i++) {
             rc = read_mpi (inp, &sk->mpi[i], 1);
             if (rc)
@@ -485,6 +495,8 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
         if (rc)
             return CDK_Inv_Packet;
         nskey = cdk_pk_get_nskey (sk->pk->pubkey_algo);
+        if( !nskey )
+            return CDK_Inv_Algo;
         for (i = 0; i < nskey; i++)
             sk->mpi[i] = NULL;
         sk->is_protected = 1;
@@ -496,14 +508,14 @@ read_secret_key (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
 
 
 static int
-read_secret_subkey (CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk)
+read_secret_subkey( CDK_STREAM inp, size_t pktlen, cdkPKT_secret_key * sk )
 {
     int rc = 0;
 
-    if (!inp || !sk || !sk->pk)
+    if( !inp || !sk || !sk->pk )
         return CDK_Inv_Value;
 
-    rc = read_secret_key (inp, pktlen, sk);
+    rc = read_secret_key( inp, pktlen, sk );
     sk->is_primary = 0;
     return rc;
 }
@@ -668,56 +680,59 @@ read_onepass_sig (CDK_STREAM inp, size_t pktlen, cdkPKT_onepass_sig * sig)
 
 
 static int
-read_signature (CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig)
+read_signature( CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig )
 {
     struct cdk_subpkt_s * node = NULL;
     size_t nbytes;
-    int i, size;
+    int i, size, nsig;
     int rc = 0;
 
-    if (!inp || !sig)
+    if( !inp || !sig )
         return CDK_Inv_Value;
 
-    if (DEBUG_PKT)
-        _cdk_log_debug ("** read signature packet\n");
+    if( DEBUG_PKT )
+        _cdk_log_debug( "** read signature packet\n" );
 
-    if (pktlen < 10)
+    if( pktlen < 10 )
         return CDK_Inv_Packet;
-    sig->version = stream_getc (inp);
-    if (sig->version < 2 || sig->version > 4)
+    sig->version = stream_getc( inp );
+    if( sig->version < 2 || sig->version > 4 )
         return CDK_Inv_Packet_Ver;
 
     sig->flags.exportable = 1;
     sig->flags.revocable = 1;
   
-    if (sig->version < 4) {
-        if (stream_getc (inp) != 5)
+    if( sig->version < 4 ) {
+        if( stream_getc( inp ) != 5 )
             return CDK_Inv_Packet;
 
-        sig->sig_class = stream_getc (inp);
-        sig->timestamp = read_32 (inp);
-        sig->keyid[0] = read_32 (inp);
-        sig->keyid[1] = read_32 (inp);
-        sig->pubkey_algo = stream_getc (inp);
-        sig->digest_algo = stream_getc (inp);
-        sig->digest_start[0] = stream_getc (inp);
-        sig->digest_start[1] = stream_getc (inp);
-        for (i = 0; i < cdk_pk_get_nsig (sig->pubkey_algo); i++) {
-            rc = read_mpi (inp, &sig->mpi[i], 0);
-            if (rc)
+        sig->sig_class = stream_getc( inp );
+        sig->timestamp = read_32( inp );
+        sig->keyid[0] = read_32( inp );
+        sig->keyid[1] = read_32( inp );
+        sig->pubkey_algo = stream_getc( inp );
+        sig->digest_algo = stream_getc( inp );
+        sig->digest_start[0] = stream_getc( inp );
+        sig->digest_start[1] = stream_getc( inp );
+        nsig = cdk_pk_get_nsig( sig->pubkey_algo );
+        if( !nsig )
+            return CDK_Inv_Algo;
+        for( i = 0; i < nsig; i++ ) {
+            rc = read_mpi( inp, &sig->mpi[i], 0 );
+            if( rc )
                 break;
 	}
     }
     else {
-        sig->sig_class = stream_getc (inp);
-        sig->pubkey_algo = stream_getc (inp);
-        sig->digest_algo = stream_getc (inp);
-        sig->hashed_size = read_16 (inp);
+        sig->sig_class = stream_getc( inp );
+        sig->pubkey_algo = stream_getc( inp );
+        sig->digest_algo = stream_getc( inp );
+        sig->hashed_size = read_16( inp );
         size = sig->hashed_size;
         sig->hashed = NULL;
-        while (size > 0) {
-            rc = read_subpkt (inp, &sig->hashed, &nbytes);
-            if (rc)
+        while( size > 0 ) {
+            rc = read_subpkt( inp, &sig->hashed, &nbytes );
+            if( rc )
                 break;
             size -= nbytes;
         }
@@ -761,11 +776,14 @@ read_signature (CDK_STREAM inp, size_t pktlen, cdkPKT_signature * sig)
             else if (node->type == SIGSUBPKT_EXPORTABLE && node->d[0] == 0)
                 sig->flags.exportable = 0;
 	}
-        sig->digest_start[0] = stream_getc (inp);
-        sig->digest_start[1] = stream_getc (inp);
-        for (i = 0; i < cdk_pk_get_nsig (sig->pubkey_algo); i++) {
-            rc = read_mpi (inp, &sig->mpi[i], 0);
-            if (rc)
+        sig->digest_start[0] = stream_getc( inp );
+        sig->digest_start[1] = stream_getc( inp );
+        nsig = cdk_pk_get_nsig( sig->pubkey_algo );
+        if( !nsig )
+            return CDK_Inv_Algo;
+        for( i = 0; i < nsig; i++ ) {
+            rc = read_mpi( inp, &sig->mpi[i], 0 );
+            if( rc )
                 break;
 	}
     }
@@ -869,6 +887,13 @@ skip_packet (CDK_STREAM inp, size_t pktlen)
 }
 
 
+/**
+ * cdk_pkt_parse:
+ * @inp: the input stream
+ * @pkt: allocated packet handle to store the packet
+ *
+ * Parse the next packet on the @inp stream and return its contents in @pkt.
+ **/
 int
 cdk_pkt_parse (CDK_STREAM inp, CDK_PACKET * pkt)
 {
