@@ -1,6 +1,6 @@
 /* -*- Mode: C; c-file-style: "bsd" -*- */
 /* sign.c - Signing routines
- *        Copyright (C) 2002 Timo Schulz
+ *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -34,7 +34,8 @@
 
 
 static int file_clearsign( CDK_HD, CDK_STRLIST, const char *, const char * );
-
+static int stream_clearsign( CDK_HD, CDK_STREAM, CDK_STREAM, CDK_STRLIST );
+    
 
 static void
 sig_calc_subpkt_size( cdkPKT_signature * sig )
@@ -141,6 +142,8 @@ write_plaintext( CDK_STREAM inp, CDK_STREAM out )
         return CDK_Inv_Value;
   
     cdk_stream_seek( inp, 0 );
+    if( !s )
+        s = "_CONSOLE";
     pt = cdk_calloc( 1, sizeof *pt + strlen( s ) + 1 );
     if( !pt )
         return CDK_Out_Of_Core;
@@ -196,7 +199,7 @@ sig_get_version( CDK_HD hd, CDK_KEYLIST kl )
   
     for( l = kl; l; l = l->next ) {
         if( (l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3)
-            ||( l->type == CDK_PKT_SECRET_KEY && l->key.sk->version == 3))
+            || (l->type == CDK_PKT_SECRET_KEY && l->key.sk->version == 3))
             return 3;
     }
     return 4;
@@ -222,7 +225,78 @@ sig_write_old( CDK_KEYLIST list, CDK_STREAM inp, CDK_STREAM out,
     
 
 /**
- * cdk_file_sign: Sign a file.
+ * cdk_stream_sign:
+ * @hd: session handle
+ * @inp: input stream
+ * @out: output stream
+ * @locusr: local user list for signing
+ * @remustr: remote user list for encrypting
+ * @encryptflag: shall the output be encrypted? (1/0)
+ * @sigmode: signature mode
+ *
+ **/
+int
+cdk_stream_sign( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out,
+                 CDK_STRLIST locusr, CDK_STRLIST remusr,
+                 int encryptflag, int sigmode )
+{
+    CDK_KEYLIST list;
+    cdkPKT_secret_key * sk;
+    md_filter_s * mfx;
+    int sigver, digest_algo;
+    int rc, detached = sigmode == CDK_SIGMODE_DETACHED;
+
+    if( sigmode == CDK_SIGMODE_CLEAR )
+        return stream_clearsign( hd, inp, out, locusr );
+    
+    rc = cdk_sklist_build( hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN );
+    if( rc )
+        return rc;
+
+    sk = list->key.sk;
+    digest_algo = _cdk_sig_hash_for( sk->pubkey_algo, sk->version );
+
+    if( hd->opt.armor )
+        cdk_stream_set_armor_flag( out, detached? 3 : 0 );
+  
+    if( encryptflag ) {
+        cdk_stream_set_cache( out, 1 );
+        rc = write_pubkey_enc_list( hd, out, remusr );
+        if( rc ) {
+            cdk_sklist_release( list );
+            return rc;
+        }
+        cdk_stream_set_cipher_flag( out, hd->dek, hd->dek->use_mdc );
+        cdk_stream_set_cache( out, 0 );
+    }
+  
+    cdk_stream_set_hash_flag( inp, digest_algo );
+    /* kick off the filter */
+    sigver = sig_get_version( hd, list );
+    cdk_stream_read( inp, NULL, 0 );
+    mfx = _cdk_stream_get_opaque( inp, fHASH );
+    if( mfx && mfx->md ) {
+        if( sigver == 3 ) {
+            rc = sig_write_old( list, inp, out, mfx->md, detached );
+            cdk_sklist_release( list );
+            return rc;
+        }
+        if( !detached ) {
+            rc = cdk_sklist_write_onepass( list, out, 0x00 );
+            if( !rc )
+                rc = write_plaintext( inp, out );
+        }
+        if( !rc )
+            rc = cdk_sklist_write( list, out, mfx->md, 0x00, sigver );
+    }
+
+    cdk_sklist_release( list );
+    return rc;
+}
+
+
+/**
+ * cdk_file_sign:
  * @locusr: List of userid which should be used for signing
  * @remusr: If encrypt is valid, the list of recipients
  * @file: Name of the input file
@@ -230,6 +304,7 @@ sig_write_old( CDK_KEYLIST list, CDK_STREAM inp, CDK_STREAM out,
  * @sigmode: Signature mode
  * @encrypt: enable sign and encrypt
  *
+ * Sign a file.
  **/
 int
 cdk_file_sign( CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
@@ -237,11 +312,7 @@ cdk_file_sign( CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
                int sigmode, int encryptflag )
 {
     CDK_STREAM inp = NULL, out = NULL;
-    CDK_KEYLIST list;
-    cdkPKT_secret_key * sk;
-    md_filter_s * mfx;
-    int sigver, digest_algo;
-    int rc = 0, detached = sigmode == CDK_SIGMODE_DETACHED;
+    int rc = 0;
 
     if( !file || !output )
         return CDK_Inv_Value;
@@ -267,51 +338,11 @@ cdk_file_sign( CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
         return rc;
     }
 
-    rc = cdk_sklist_build( hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN );
-    if( rc ) {
-        cdk_stream_close( inp );
-        cdk_stream_close( out );
-        return rc;
-    }
-
-    sk = list->key.sk;
-    digest_algo = _cdk_sig_hash_for( sk->pubkey_algo, sk->version );
-
-    if( hd->opt.armor )
-        cdk_stream_set_armor_flag( out, 0 );
-  
-    if( encryptflag ) {
-        cdk_stream_set_cache( out, 1 );
-        rc = write_pubkey_enc_list( hd, out, remusr );
-        if( rc )
-            goto leave;
-        cdk_stream_set_cipher_flag( out, hd->dek, hd->dek->use_mdc );
-        cdk_stream_set_cache( out, 0 );
-    }
-  
-    cdk_stream_set_hash_flag( inp, digest_algo );
-    /* kick off the filter */
-    sigver = sig_get_version( hd, list );
-    cdk_stream_read( inp, NULL, 0 );
-    mfx = _cdk_stream_get_opaque( inp, fHASH );
-    if( mfx && mfx->md ) {
-        if( sigver == 3 ) {
-            rc = sig_write_old( list, inp, out, mfx->md, detached );
-            goto leave;
-        }
-        if( !detached ) {
-            rc = cdk_sklist_write_onepass( list, out, 0x00 );
-            if( !rc )
-                rc = write_plaintext( inp, out );
-        }
-        if( !rc )
-            rc = cdk_sklist_write( list, out, mfx->md, 0x00, sigver );
-    }
-
- leave:
-    cdk_sklist_release( list );
+    rc = cdk_stream_sign( hd, inp, out, locusr, remusr, encryptflag, sigmode );
+    
     cdk_stream_close( inp );
     cdk_stream_close( out );
+    
     return rc;
 }
 
@@ -350,46 +381,28 @@ _cdk_trim_string( char * s, int canon )
 
 
 static int
-file_clearsign( CDK_HD hd, CDK_STRLIST locusr,
-                const char * file, const char * output )
+stream_clearsign( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out,
+                  CDK_STRLIST locusr )
 {
     GCRY_MD_HD md = NULL;
-    CDK_STREAM inp = NULL, out = NULL, tmp;
     CDK_KEYLIST list;
+    CDK_STREAM tmp;
     cdkPKT_secret_key * sk;
     const char * s;
-    int digest_algo, sigver;
     char buf[1024+2];
-    int rc, nread = 0;
-  
-    if( !locusr || !file || !output )
-        return CDK_Inv_Value;
-    if( !hd->opt.overwrite && _cdk_check_file( output ) )
-        return CDK_Inv_Mode;
-
-    rc = cdk_stream_open( file, &inp );
+    int digest_algo, sigver;
+    int rc, nread;
+    
+    rc = cdk_sklist_build( hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN );
     if( rc )
         return rc;
-  
-    rc = cdk_stream_create( output, &out );
-    if( rc ) {
-        cdk_stream_close( inp );
-        return rc;
-    }
-
-    rc = cdk_sklist_build( hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN );
-    if( rc ) {
-        cdk_stream_close( inp );
-        cdk_stream_close( out );
-        return rc;
-    }
 
     sk = list->key.sk;
     digest_algo = _cdk_sig_hash_for( sk->pubkey_algo, sk->version );
     md = gcry_md_open( digest_algo, 0 );
     if( !md ) {
-        rc = CDK_Gcry_Error;
-        goto leave;
+        cdk_sklist_release( list );
+        return CDK_Gcry_Error;
     }
 
     s = _cdk_armor_get_lineend( );
@@ -400,7 +413,7 @@ file_clearsign( CDK_HD hd, CDK_STRLIST locusr,
   
     while( !cdk_stream_eof( inp ) ) {
         nread = _cdk_stream_gets( inp, buf, sizeof buf-1 );
-        if( !nread)
+        if( !nread )
             break;
         _cdk_trim_string( buf, 1 );
         gcry_md_write( md, buf, strlen( buf ) );
@@ -415,7 +428,7 @@ file_clearsign( CDK_HD hd, CDK_STRLIST locusr,
         _cdk_stream_puts( out, buf );
     }
     _cdk_stream_puts( out, s );
-    tmp = cdk_stream_tmp(  );
+    tmp = cdk_stream_tmp( );
     if( !tmp ) {
         rc = CDK_Out_Of_Core;
         goto leave;
@@ -445,8 +458,37 @@ file_clearsign( CDK_HD hd, CDK_STRLIST locusr,
  leave:
     gcry_md_close( md );
     cdk_sklist_release( list );
+    return rc;
+}
+
+
+static int
+file_clearsign( CDK_HD hd, CDK_STRLIST locusr,
+                const char * file, const char * output )
+{
+    CDK_STREAM inp = NULL, out = NULL;
+    int rc;
+  
+    if( !locusr || !file || !output )
+        return CDK_Inv_Value;
+    if( !hd->opt.overwrite && _cdk_check_file( output ) )
+        return CDK_Inv_Mode;
+
+    rc = cdk_stream_open( file, &inp );
+    if( rc )
+        return rc;
+  
+    rc = cdk_stream_create( output, &out );
+    if( rc ) {
+        cdk_stream_close( inp );
+        return rc;
+    }
+
+    rc = stream_clearsign( hd, inp, out, locusr );
+    
     cdk_stream_close( inp );
     cdk_stream_close( out );
+    
     return rc;
 }
 
