@@ -63,11 +63,12 @@ cdk_stream_open( const char * file, CDK_STREAM * ret_s )
         return CDK_Out_Of_Core;
     s->fname = cdk_strdup( file );
     if( !s->fname ) {
-        cdk_free( s  );
+        cdk_free( s );
         return CDK_Out_Of_Core;
     }
     s->fp = fopen( file, "rb" );
     if( !s->fp ) {
+        cdk_free( s->fname );
         cdk_free( s );
         return CDK_File_Error;
     }
@@ -99,13 +100,12 @@ cdk_stream_new( const char * file, CDK_STREAM * ret_s )
     s->flags.write = 1;
     if( !file ) {
         s->flags.temp = 1;
-        s->fp = tmpfile(  );
+        s->fp = tmpfile( );
         if( !s->fp ) {
             cdk_free( s );
             return CDK_File_Error;
         }
-        if( ret_s)
-            *ret_s = s;
+        *ret_s = s;
         return 0;
     }
     s->fname = cdk_strdup( file );
@@ -133,7 +133,7 @@ cdk_stream_new( const char * file, CDK_STREAM * ret_s )
  * this kind of stream and everything is written directly to the stream.
  **/
 int
-cdk_stream_create( const char * file, CDK_STREAM *ret_s )
+cdk_stream_create( const char * file, CDK_STREAM * ret_s )
 {
     CDK_STREAM s;
 
@@ -223,7 +223,7 @@ _cdk_stream_append( const char * file, CDK_STREAM * ret_s )
         return CDK_Inv_Value;
     rc = cdk_stream_open( file, &s );
     if( rc )
-        return rc;
+        return rc;    
     fp = fopen( file, "a+b" );
     if( !fp ) {
         cdk_stream_close( s );
@@ -237,7 +237,6 @@ _cdk_stream_append( const char * file, CDK_STREAM * ret_s )
 }
 
       
-
 int
 cdk_stream_control( CDK_STREAM s, int ctl, int val )
 {
@@ -306,6 +305,7 @@ cdk_stream_close( CDK_STREAM s )
         return CDK_Inv_Value;
     
     _cdk_log_debug( "close stream `%s'\n", s->fname? s->fname : "[temp]" );
+    
     if( !s->flags.filtrated && !s->error )
         rc = cdk_stream_flush( s );
     if( s->fname || s->flags.temp ) {
@@ -317,7 +317,8 @@ cdk_stream_close( CDK_STREAM s )
     f = s->filters;
     while( f ) {
         f2 = f->next;
-        f->fnc( f->opaque, STREAMCTL_FREE, NULL, NULL );
+        if( f->fnc )
+            f->fnc( f->opaque, STREAMCTL_FREE, NULL, NULL );
         cdk_free( f );
         f = f2;
     }
@@ -379,6 +380,7 @@ cdk_stream_get_length( CDK_STREAM s )
 
     if( !s )
         return (unsigned)-1;
+    
     rc = stream_flush( s );
     if( rc ) {
         s->error = CDK_File_Error;
@@ -430,7 +432,7 @@ stream_filter_add( CDK_STREAM s, FILTER_FUNC fnc, int type)
         return flt;
   
     flt = filter_add( s );
-    if( !flt)
+    if( !flt )
         return NULL;
     flt->fnc = fnc;
     flt->flags.enabled = 1;
@@ -544,16 +546,14 @@ stream_filter_write( CDK_STREAM s )
         }
         rc = f->fnc( f->opaque, f->ctl, s->fp, f->tmp );
         _cdk_log_debug( "filter [write]: type=%d rc=%d\n", f->type, rc );
+        if( !rc )
+            rc = stream_fp_replace( s, &f->tmp );
+        if( !rc )
+            rc = cdk_stream_seek( s, 0 );
         if( rc ) {
             fclose( f->tmp );
             break;
         }
-        rc = stream_fp_replace( s, &f->tmp );
-        if( rc )
-            break;
-        rc = cdk_stream_seek( s, 0 );
-        if( rc )
-            break;
     }
     return rc;
 }
@@ -582,7 +582,8 @@ stream_filter_read( CDK_STREAM s )
             break;
         }
         rc = f->fnc( f->opaque, f->ctl, s->fp, f->tmp );
-        _cdk_log_debug( "filter [read]: type=%d rc=%d\n", f->type, rc );
+        _cdk_log_debug( "filter %s [read]: type=%d rc=%d\n",
+                        s->fname? s->fname : "[temp]", f->type, rc );
         if( rc )
             break;
       
@@ -614,6 +615,7 @@ _cdk_stream_get_opaque( CDK_STREAM s, int id )
 
     if( !s )
         return NULL;
+    
     for( f = s->filters; f; f = f->next ) {
         if( f->type == id )
             return f->opaque;
@@ -639,7 +641,8 @@ cdk_stream_read( CDK_STREAM s, void * buf, size_t count )
     int rc;
 
     if( !s )
-        return -1;
+        return EOF;
+    
     if( s->flags.write && !s->flags.temp )
         return EOF; /* this is a write stream */
   
@@ -669,11 +672,12 @@ cdk_stream_getc( CDK_STREAM s )
     int nread;
 
     if( !s )
-        return -1;
+        return EOF;
+    
     nread = cdk_stream_read( s, buf, 1 );
     if( nread == EOF ) {
         s->error = CDK_File_Error;
-        return -1;
+        return EOF;
     }
     return buf[0];
 }
@@ -696,6 +700,7 @@ cdk_stream_write( CDK_STREAM s, const void * buf, size_t count )
 
     if( !s )
         return CDK_Inv_Value;
+    
     if( !s->flags.write )
         return CDK_Inv_Mode; /* this is a read stream */
 
@@ -703,7 +708,7 @@ cdk_stream_write( CDK_STREAM s, const void * buf, size_t count )
         return stream_flush( s  );
 
     if( s->cache.on ) {
-        if( s->cache.size + count > sizeof( s->cache.buf))
+        if( s->cache.size + count > sizeof( s->cache.buf ) )
             return CDK_EOF;
         memcpy( s->cache.buf + s->cache.size, buf, count  );
         s->cache.size += count;
@@ -724,12 +729,12 @@ cdk_stream_putc( CDK_STREAM s, int c )
     int nwritten;
 
     if( !s )
-        return -1;
+        return EOF;
     buf[0] = c;
     nwritten = cdk_stream_write( s, buf, 1 );
     if( nwritten == EOF ) {
         s->error = CDK_File_Error;
-        return -1;
+        return EOF;
     }
     return 0;
 }
@@ -748,7 +753,7 @@ cdk_stream_seek( CDK_STREAM s, long offset )
     int rc;
 
     if( !s )
-        return CDK_Inv_Value;
+        return CDK_Inv_Value;    
     if( offset < cdk_stream_get_length( s ) )
         s->flags.eof = 0;
     rc = fseek( s->fp, offset, SEEK_SET );
@@ -866,7 +871,7 @@ cdk_stream_set_hash_flag( CDK_STREAM s, int algo )
     struct stream_filter_s * flt;
 
     if( !s )
-        return CDK_Inv_Value;
+        return CDK_Inv_Value;    
     if( stream_get_mode( s ) )
         return CDK_Inv_Mode;
     flt = stream_filter_add( s, _cdk_filter_hash, fHASH );
@@ -921,10 +926,10 @@ cdk_stream_kick_off( CDK_STREAM inp, CDK_STREAM out )
         return CDK_Inv_Value;
     while( !cdk_stream_eof( inp ) ) {
         nread = cdk_stream_read( inp, buf, sizeof buf-1 );
-        if( nread == EOF)
+        if( nread == EOF )
             break;
         nwritten = cdk_stream_write( out, buf, nread );
-        if( nwritten == EOF)
+        if( nwritten == EOF )
             rc = CDK_File_Error;
     }
     wipemem( buf, sizeof buf );
@@ -940,7 +945,8 @@ _cdk_stream_gets( CDK_STREAM s, char * buf, size_t count )
     if( !s )
         return CDK_Inv_Value;
     while( !cdk_stream_eof( s ) && count > 0 ) {
-        if( (c = cdk_stream_getc( s )) == EOF || c == '\r' || c == '\n' ) {
+        c = cdk_stream_getc( s );
+        if( c == EOF || c == '\r' || c == '\n' ) {
             buf[i++] = '\0';
             break;
         }
@@ -967,4 +973,3 @@ _cdk_stream_set_blockmode( CDK_STREAM s, size_t nbytes )
     s->blkmode = nbytes;  
     return 0;
 }
-
