@@ -61,6 +61,25 @@ stream_peek( CDK_STREAM inp, char *s, size_t count )
 }
 
 
+int
+cdk_stream_verify( CDK_HD hd, CDK_STREAM inp, CDK_STREAM out )
+{
+    if( cdk_armor_filter_use( inp ) ) {
+        cdk_stream_set_armor_flag( inp, 0 );
+        cdk_stream_control( inp, CDK_STREAMCTL_RESET, 1 );
+    }
+    return _cdk_proc_packets( hd, inp, NULL, NULL, NULL );
+}
+    
+
+/**
+ * cdk_file_verify:
+ * @hd: the session handle
+ * @file: the input file
+ * @output: the output file
+ *
+ * Verify a signature.
+ **/
 int
 cdk_file_verify( CDK_HD hd, const char * file, const char * output )
 {
@@ -82,14 +101,14 @@ cdk_file_verify( CDK_HD hd, const char * file, const char * output )
             return CDK_EOF;
         buf[n] = '\0';
         if( strstr( buf, "BEGIN PGP SIGNED MESSAGE" ) ) {
-            rc = file_verify_clearsign( hd, file, output );
             cdk_stream_close( inp );
+            rc = file_verify_clearsign( hd, file, output );
             return rc;
         }
-        else
-            cdk_stream_set_armor_flag( inp, 0 );
+        cdk_stream_set_armor_flag( inp, 0 );
+        cdk_stream_control( inp, CDK_STREAMCTL_RESET, 1 );
     }
-    rc = _cdk_proc_packets( hd, inp, NULL, NULL );
+    rc = _cdk_proc_packets( hd, inp, NULL, NULL, NULL );
     cdk_stream_close( inp );
     return rc;
 }
@@ -118,6 +137,60 @@ _cdk_result_verify_new( void )
 }
 
 
+/**
+ * cdk_sig_get_info_as_xml:
+ * @hd: session handle
+ * @advanced: flag to indicate whether to include extended data
+ *
+ * Return an XML fragment which contains the status of the verification
+ * procedure. If no signature is present, the fragment is empty.
+ **/
+byte *
+cdk_sig_get_info_as_xml( CDK_HD hd, int advanced )
+{
+    char * p;
+    const char * fmt;
+    const u32 * keyid;
+    
+    if( !hd || !cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_STATUS ) )
+        return cdk_strdup( "<OpenPGP-Signature-Status>"
+                           "</OpenPGP-Signature-Status>" );
+    fmt = "<OpenPGP-Signature>\n"
+          "  <Version>%d</Version>\n"
+          "  <Created>%lu</Created>\n"
+          "  <Publickey-Algo>%d</Publickey-Algo>\n"
+          "  <Digest-Algo>%d</Digest-Algo>\n"
+          "  <KeyID>%08lX%08lX</KeyID>\n"
+          "  <Expired>%d</Expired>\n"
+          "  <Status>%d</Status>\n"
+          "</OpenPGP-Signature>\n";
+    p = cdk_calloc( 1, strlen( fmt ) + 512 );
+    if( !p )
+        return NULL;
+    keyid = cdk_sig_get_data_attr( hd, 0, CDK_ATTR_KEYID );
+    sprintf( p, fmt,
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_VERSION ),
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_CREATED ),
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_ALGO_PK ),
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_ALGO_MD ),
+             keyid[0], keyid[1],
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_FLAGS )
+             & CDK_FLAG_SIG_EXPIRED? 1 : 0,
+             cdk_sig_get_ulong_attr( hd, 0, CDK_ATTR_STATUS )
+        );
+    return p;
+}
+
+
+/**
+ * cdk_sig_get_ulong_attr:
+ * @hd: session handle
+ * @idx: index of the signature
+ * @what: attribute id
+ *
+ * Extract the requested attribute of the signature. The returned value
+ * is always an integer (max. 32-bit).
+ **/
 u32
 cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what)
 {
@@ -146,6 +219,15 @@ cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what)
 }
 
 
+/**
+ * cdk_sig_get_data_attr:
+ * @hd: session handle
+ * @idx: index of the signature
+ * @what: attribute id.
+ *
+ * Extract the requested attribute of the signature. The returned value
+ * is always a constant object to the data.
+ **/
 const void *
 cdk_sig_get_data_attr( CDK_HD hd, int idx, int what )
 {
@@ -279,7 +361,7 @@ file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
     cdk_stream_set_armor_flag (tmp, 0);
     cdk_stream_read (tmp, NULL, 0);
 
-    rc = _cdk_proc_packets( hd, tmp, NULL, md );
+    rc = _cdk_proc_packets( hd, tmp, NULL, NULL, md );
 
  leave:
     cdk_stream_close (out);
