@@ -1,5 +1,6 @@
-/* minpg.c - Minimal Privacy Guard
- *        Copyright (C) 2002 Timo Schulz
+/* -*- Mode: C; c-file-style: "gnu" -*- */
+/* minpg.c - Minimal Privacy Guard (MinPG)
+ *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -18,6 +19,7 @@
  * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * This is only a regression test and should _NOT_ used for real work.
+ * But on the other hand, it is _not_ insecure.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,6 +33,7 @@
 #include <time.h>
 #include <stdlib.h>
 
+
 enum
 {
   cSignEncr = 'b',
@@ -44,9 +47,12 @@ enum
   cGenKey = 'w',
   cSigList = 'l',
   cEncr = 'e',
+  cEncrFiles = '@',
   cDecr = 'd',
+  cDecrFiles = '$',
   cExport = 'x',
   cImport = 'i',
+  cShowpref = 'q',
   oCompress = 'z',
   oRecipient = 'r',
   oVerbose = 'v',
@@ -59,6 +65,7 @@ enum
   oArmor = 'p',
   oCompat = 'g',
   oNoCompress = '0',
+  oDetachSign = 'j',  
 };
 
 
@@ -66,18 +73,17 @@ static struct
 {
   const char * name;
   int algid;
-} algo_table[] =
-{
-  {"3DES", GCRY_CIPHER_3DES},
-  {"CAST5", GCRY_CIPHER_CAST5},
-  {"BLOWFISH", GCRY_CIPHER_BLOWFISH},
-  {"TWOFISH", GCRY_CIPHER_TWOFISH},
-  {"AES", GCRY_CIPHER_AES128},
-  {"AES192", GCRY_CIPHER_AES192},
-  {"AES256", GCRY_CIPHER_AES256},
-  {"MD5", GCRY_MD_MD5},
-  {"SHA1", GCRY_MD_SHA1},
-  {"RIPEMD160", GCRY_MD_RMD160},
+} algo_table[] = {
+  {"3DES", CDK_CIPHER_3DES},
+  {"CAST5", CDK_CIPHER_CAST5},
+  {"BLOWFISH", CDK_CIPHER_BLOWFISH},
+  {"TWOFISH", CDK_CIPHER_TWOFISH},
+  {"AES", CDK_CIPHER_AES},
+  {"AES192", CDK_CIPHER_AES192},
+  {"AES256", CDK_CIPHER_AES256},
+  {"MD5", CDK_MD_MD5},
+  {"SHA1", CDK_MD_SHA1},
+  {"RIPEMD160", CDK_MD_RMD160},
   {NULL},
 };
 
@@ -87,6 +93,7 @@ char * outname = "(none)";
 char * cipher_algo = "(none)";
 char * digest_algo = "(none)";
 char * ks_keyid = "(none)";
+char * compress_level = "(none)";
 int cipher_algid = 0;
 int digest_algid = 0;
 int use_outname = 0;
@@ -94,6 +101,10 @@ int use_outname = 0;
 
 struct poptOption arguments[] =
 {
+  {"show-prefs", cShowpref, POPT_ARG_STRING, &user, cShowpref,
+   "show the preferences of a key", 0},
+  {"detach-sign", oDetachSign, POPT_ARG_NONE, 0, oDetachSign,
+   "create a detached signature", 0},
   {"gen-key", cGenKey, POPT_ARG_NONE, 0, cGenKey, "Generate a key", 0},
   {"no-compression", oNoCompress, POPT_ARG_NONE, 0, oNoCompress,
    "Turns off compression", 0},
@@ -108,9 +119,9 @@ struct poptOption arguments[] =
   {"fetch-key", oFetchKey, POPT_ARG_STRING, &ks_keyid, oFetchKey,
    "Fetch key from keyserver", 0},
   {"cipher-algo", oCipherAlg, POPT_ARG_STRING, &cipher_algo, oCipherAlg,
-   "Cipher Algo", 0},
+   "Cipher algorithm", 0},
   {"digest-algo", oDigestAlgo, POPT_ARG_STRING, &digest_algo, oDigestAlgo,
-   "Digest Algo", 0},
+   "Digest algorithm", 0},
   {"list-sigs", cSigList, POPT_ARG_STRING, &user, cSigList,
    "List signatures", 0},
   {"output", oOutput, POPT_ARG_STRING, &outname, oOutput,
@@ -127,14 +138,18 @@ struct poptOption arguments[] =
    "Protect a file with ASCII-Armor", 0},  
   {"dearmor", cDearmor, POPT_ARG_STRING, &file, cDearmor,
    "Remove ASCII-Armor from a file", 0},
-  {"compress", oCompress, POPT_ARG_NONE, 0, oCompress,
+  {"compress", oCompress, POPT_ARG_STRING, &compress_level, oCompress,
    "Use compression", 0},
   {"symmetric", cSymmetric, POPT_ARG_STRING, &file, cSymmetric,
    "Symmetrically encrypt a file", 0},
   {"recipient", oRecipient, POPT_ARG_STRING, &user, oRecipient,
    "Use this recipient", 0},
   {"encrypt", cEncr, POPT_ARG_STRING, &file, cEncr, "Encrypt a file", 0},
+  {"encrypt-files", cEncrFiles, POPT_ARG_STRING, &file, cEncrFiles,
+   "Encrypt |files|", 0},
   {"decrypt", cDecr, POPT_ARG_STRING, &file, cDecr, "Decrypt a file", 0},
+  {"decrypt-files", cDecrFiles, POPT_ARG_STRING, &file, cDecrFiles,
+   "Decrypt |files|", 0},
   {"verbose", oVerbose, POPT_ARG_NONE, 0, oVerbose, "Verbosity", 0},
   {"armor", oArmor, POPT_ARG_NONE, 0, oArmor, "Armor output", 0},
   {NULL}
@@ -209,11 +224,12 @@ key_get_letter (int pubkey_algo)
 {
   switch (pubkey_algo)
     {
-    case GCRY_PK_RSA:   return 'R';
-    case GCRY_PK_RSA_E: return 'r';
-    case GCRY_PK_ELG_E: return 'g';
-    case GCRY_PK_ELG:   return 'G';
-    case GCRY_PK_DSA:   return 'D';
+    case CDK_PK_RSA:   return 'R';
+    case CDK_PK_RSA_E: return 'r';
+    case CDK_PK_RSA_S: return 'r';
+    case CDK_PK_ELG_E: return 'g';
+    case CDK_PK_ELG:   return 'G';
+    case CDK_PK_DSA:   return 'D';
     default:            return '?';
     }
   return '?';
@@ -227,8 +243,8 @@ strdate_from_long (long timestamp)
   struct tm * tm;
 
   tm = localtime (&timestamp);
-  snprintf (kdate, sizeof kdate - 1, "%04d-%02d-%02d",
-	    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
+  sprintf (kdate, "%04d-%02d-%02d",
+           tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
   return kdate;
 }
 
@@ -272,7 +288,7 @@ algo_get_id (const char *algo, int what)
       if (!strcmp (algo, algo_table[i].name))
 	return algo_table[i].algid;
     }
-  i = what? GCRY_CIPHER_3DES: GCRY_MD_SHA1;
+  i = what? CDK_CIPHER_CAST5: CDK_MD_SHA1;
   return i;
 }
 
@@ -282,6 +298,7 @@ get_keyvalid_char (cdkPKT_public_key * pk)
 {
   static char flags[2];
   memset (flags, 0, 2);
+  /*printf ("[invalid=%d]\n", pk->is_invalid);*/
   if (pk->has_expired)
     strcat (flags, "e");
   else
@@ -290,7 +307,9 @@ get_keyvalid_char (cdkPKT_public_key * pk)
     strcat (flags, "r");
   else
     strcat (flags, " ");
-  if (!pk->has_expired && !pk->is_revoked)
+  if (pk->is_invalid)
+    strcat (flags, "?");
+  else if (!pk->has_expired && !pk->is_revoked)
     strcat (flags, "!");
   else
     strcat (flags, " ");
@@ -405,6 +424,8 @@ do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
       return 0;
     }
   /* Show all keys */
+  cdk_set_log_level (CDK_LOG_DEBUG);
+  
   rc = cdk_keydb_open (hd, &inp);
   if (rc)
     {
@@ -422,7 +443,6 @@ do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
       if (rc)
         break;
     }
-  cdk_stream_close (inp);
   return 0;
 }
 
@@ -435,11 +455,11 @@ do_export_keys (CDK_HD ctx, CDK_KEYDB_HD hd, CDK_STRLIST keys, int secret)
 
   if (!use_outname)
     {
-      fprintf (stdout, "Please use --output for a file.\n");
+      fprintf (stdout, "please use --output for a file.\n");
       return CDK_General_Error;
     }
 
-  rc = cdk_stream_new (outname, &outp);
+  rc = cdk_stream_create (outname, &outp);
   if (rc)
     {
       fprintf (stdout, "can't open `%s': %s\n", outname, cdk_strerror (rc));
@@ -447,7 +467,7 @@ do_export_keys (CDK_HD ctx, CDK_KEYDB_HD hd, CDK_STRLIST keys, int secret)
     }
   if (keys)
     {
-      if (cdk_handle_get_armor (ctx))
+      if (cdk_handle_control (ctx, CDK_CTLF_GET, CDK_CTL_ARMOR))
         cdk_stream_set_armor_flag (outp, CDK_ARMOR_PUBKEY);
       rc = cdk_keydb_export (hd, outp, keys);
       if (rc)
@@ -467,7 +487,7 @@ do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
 
   if (!hd)
     {
-      fprintf (stderr, "No public keyring available.\n");
+      fprintf (stderr, "no public keyring available.\n");
       return -1;
     }
   rc = cdk_stream_open (keyfile, &s);
@@ -476,6 +496,8 @@ do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
       fprintf (stderr, "open `%s' failed: %s\n", keyfile, cdk_strerror (rc));
       return -1;
     }
+  if (cdk_armor_filter_use (s))
+    cdk_stream_set_armor_flag (s, 0);
   rc = cdk_keydb_get_keyblock (s, &node);
   if (!node)
     {
@@ -546,12 +568,13 @@ get_passphrase (const char *prompt)
 
   fprintf (stdout, "%s", prompt);
   p = cdk_calloc (1, 256);
-  if (p)
-    {
-      fgets (p, maxlen, stdin);
-      p[strlen (p) - 1] = '\0';
-      return p;
-    }
+  if (!p)
+    out_of_core ();
+
+  fgets (p, maxlen, stdin);
+  p[strlen (p) - 1] = '\0';
+  return p;
+
   return NULL;
 }
 
@@ -581,6 +604,7 @@ ks_fetch_key (const char * kid)
   kidbuf[1] = keyid >> 16;
   kidbuf[2] = keyid >>  8;
   kidbuf[3] = keyid;
+  cdk_set_log_level (CDK_LOG_DEBUG);
   printf ("fetch %08lX from the keyserver...", keyid);
   rc = cdk_keyserver_recv_key ("http://horowitz.surfnet.nl", 11371,
                                kidbuf, CDK_DBSEARCH_SHORT_KEYID, &pk);
@@ -617,12 +641,12 @@ get_sig_algo (int pubkey_algo)
 {
   switch (pubkey_algo)
     {
-    case GCRY_PK_RSA_E:
-    case GCRY_PK_RSA_S:
-    case GCRY_PK_RSA  : return "RSA";
-    case GCRY_PK_ELG  :
-    case GCRY_PK_ELG_E: return "ELG";
-    case GCRY_PK_DSA  : return "DSA";
+    case CDK_PK_RSA_E:
+    case CDK_PK_RSA_S:
+    case CDK_PK_RSA  : return "RSA";
+    case CDK_PK_ELG  :
+    case CDK_PK_ELG_E: return "ELG";
+    case CDK_PK_DSA  : return "DSA";
     default           : return "???"; 
     }
   return NULL; 
@@ -635,14 +659,33 @@ get_sig_time (unsigned long timestamp)
   struct tm * tmbuf;
 
   tmbuf = localtime (&timestamp);
-  snprintf (buf, sizeof buf-1, "%04d-%02d-%02d",
-            tmbuf->tm_year+1900, tmbuf->tm_mon+1, tmbuf->tm_mday);
+  sprintf (buf, "%04d-%02d-%02d %02d:%02d:%02d",
+           tmbuf->tm_year+1900, tmbuf->tm_mon+1, tmbuf->tm_mday,
+           tmbuf->tm_hour, tmbuf->tm_min, tmbuf->tm_sec);
   return buf;  
 }
 
 
 void
-print_verify_stats (CDK_HD hd)
+print_mpibuf (const unsigned char * buf, int pubkey_algo)
+{
+  int i, j, n;
+
+  printf ("signature data:\n");
+  for (i = 0, j = 0; i < cdk_pk_get_nsig (pubkey_algo); i++)
+    {
+      n  = buf[j++] << 8;
+      n |= buf[j++];
+      n = (n + 7) / 8;
+      while (n--)
+        printf ("%02x", buf[j++]);
+    }
+  printf ("\n");
+}
+      
+
+void
+print_verify_stats (CDK_HD hd, int extflag)
 {
   const unsigned long * keyid;
   unsigned long created;
@@ -657,6 +700,8 @@ print_verify_stats (CDK_HD hd)
       printf ("(%s) keyid %08lX%08lX algo %s created %s\n",
               get_sig_status (sigstat), keyid[0], keyid[1],
               get_sig_algo (pkalgo), get_sig_time (created));
+      if (extflag)
+        print_mpibuf (cdk_sig_get_data_attr (hd, 0, CDK_ATTR_MPI), pkalgo);
     }
 }
 
@@ -665,24 +710,43 @@ int
 genkey (void)
 {
   int c, algo = 0, rc =0;
+  unsigned char mdbuf[3] = {CDK_MD_MD5, CDK_MD_RMD160, CDK_MD_SHA1};
   CDK_KEYGEN_CTX hd;
   
   printf ("(1) DSA (sign only)\n");
+  printf ("(2) DSA and ElGamal\n");
+  printf ("(3) DSA and RSA\n");
+  printf ("(4) RSA and ElGamal\n");
+  printf ("your choice: ");
   c = fgetc (stdin);
   switch (c)
     {
-    case '1': algo = GCRY_PK_DSA; break;
-    default: printf ("invalid pk algorithm\n"); break;
+    case '1':
+    case '2': 
+    case '3': algo = CDK_PK_DSA; break;
+    case '4': algo = CDK_PK_RSA; break;
+    default: printf ("invalid public key algorithm\n"); break;
     }
   if (!algo)
     return CDK_Inv_Algo;
-  cdk_salloc (1);
+  cdk_salloc (1, 0);
   cdk_keygen_new (&hd);
   cdk_keygen_set_algo_info (hd, 0, algo, 768);
-  cdk_keygen_set_algo_info (hd, 1, GCRY_PK_ELG_E, 768);
+  if (c > '1' && c < '5')
+    {
+      switch (c)
+        {
+        case '4':
+        case '2': cdk_keygen_set_algo_info (hd, 1, CDK_PK_ELG_E, 768); break;
+        case '3': cdk_keygen_set_algo_info (hd, 1, CDK_PK_RSA_E, 768); break;
+        }
+      cdk_keygen_set_expire_date (hd, 1, 8640000);
+    }
   cdk_keygen_set_name (hd, "Otto Bismarck");
   cdk_keygen_set_mdc_feature (hd, 1);
-  cdk_keygen_set_expire_date (hd, 0, 1040820537);
+  cdk_keygen_set_expire_date (hd, 0, 864000);
+  cdk_keygen_set_prefs (hd, CDK_PREFTYPE_HASH, mdbuf, 3);
+  cdk_keygen_set_passphrase (hd, "abc");
   rc = cdk_keygen_start (hd);
   if (!rc)
     rc = cdk_keygen_save (hd, "pub.test", "sec.test");
@@ -691,6 +755,59 @@ genkey (void)
 }
 
 
+static void
+show_one_array (cdkPKT_user_id * id, int type)
+{
+  size_t n = 0, i = 0;
+  unsigned char * p;
+
+  p = cdk_prefs_get_array (id, type, &n);
+  if (p)
+    {
+      for (i = 0; i < n; i++)
+        {
+          switch (type)
+            {
+            case CDK_PREFTYPE_SYM : fputc ('S', stdout); break;
+            case CDK_PREFTYPE_HASH: fputc ('H', stdout); break;
+            case CDK_PREFTYPE_ZIP : fputc ('Z', stdout); break;
+            }
+          printf ("%d ", p[i]);
+        }
+      cdk_free (p);
+    }    
+}
+
+
+static void
+show_prefs (const char * name, CDK_KEYDB_HD db)
+{
+  CDK_KBNODE knode = NULL;
+  CDK_PACKET * pkt;
+  int rc;
+  
+  rc = cdk_keydb_get_bypattern (db, name, &knode);
+  if (rc)
+      printf ("keydb get error %s: `%s'\n", name, cdk_strerror (rc));
+  else
+    {
+      pkt = cdk_kbnode_find_packet (knode, CDK_PKT_USER_ID);
+      if (!pkt)
+        {
+          printf ("no userid not found!\n");
+          goto leave;
+        }
+      printf ("%s preferences\n", pkt->pkt.user_id->name);
+      show_one_array (pkt->pkt.user_id, CDK_PREFTYPE_SYM);
+      show_one_array (pkt->pkt.user_id, CDK_PREFTYPE_HASH);
+      show_one_array (pkt->pkt.user_id, CDK_PREFTYPE_ZIP);
+      printf ("\n");
+    }
+ leave:
+  cdk_kbnode_release (knode);
+}
+
+    
 void
 file_error (const char * fil, const char * fnc, int rc)
 {
@@ -707,10 +824,11 @@ main (int argc, char ** argv)
   poptContext opt_ctx;
   char * output = NULL, * s;
   char u[128];
-  int c = 0, verbose = 0, i = 0;
-  int rc = 0;
+  int c = 0, verbose = 0, i = 0, sigmode = 0, zip_level = 6;
+  int rc = 0, nfiles = 0;
 
   s = "/home/twoaday/.gnupg/pubring.cdk";
+  /*s = "pubring.gpg";*/
   rc = cdk_keydb_new (&pub_db, CDK_DBTYPE_PK_KEYRING, s, strlen (s));
   if (rc)
     {
@@ -718,6 +836,7 @@ main (int argc, char ** argv)
       return 0;
     }
   s = "sec-with-pwd.gpg";
+  /*s = "secring.gpg";*/
   rc = cdk_keydb_new (&sec_db, CDK_DBTYPE_SK_KEYRING, s, strlen (s));
   if (rc)
     {
@@ -729,6 +848,9 @@ main (int argc, char ** argv)
 
   cdk_handle_new (&hd);
   cdk_handle_set_callback (hd, callback, NULL);
+  cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_OVERWRITE, 1);
+  cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_COMPRESS, 2, 5);
+  cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_KEYCACHE_ON, 1);
   /*cdk_set_progress_handler (progr_callback, NULL);*/
   opt_ctx = poptGetContext ("pred", argc, argv, arguments, 0);
   while ((c = poptGetNextOpt (opt_ctx)) >= 0)
@@ -740,21 +862,27 @@ main (int argc, char ** argv)
 	  break;
 
         case oNoCompress:
-          cdk_handle_set_compress (hd, 0);
+          if (verbose)
+            fprintf (stdout, "disable compression\n");
+          cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_COMPRESS, 0, 0);
           break;
           
         case 'p':
-          cdk_handle_set_armor (hd, 1);
+          if (verbose)
+            fprintf (stdout, "protect output with ascii armor\n");
+          cdk_handle_set_armor( hd, 1 );
           break;
 
 	case 'o':
 	  if (verbose)
-	    fprintf (stdout, "Output filename `%s'\n", outname);
+	    fprintf (stdout, "output filename `%s'\n", outname);
 	  use_outname = 1;
 	  break;
 
         case 'g':
-          cdk_handle_set_compat (hd, -1);
+          if (verbose)
+            fprintf (stdout, "set PGP compat mode\n");
+          cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_COMPAT, 1);
           break;
 
 	case 'h':
@@ -765,38 +893,49 @@ main (int argc, char ** argv)
 	case 'V':
 	  fprintf (stderr, "Minimal Privacy Guard (minpg) %s\n\n", VERSION);
 	  fprintf (stderr, "Supported algorithms:\n");
-	  fprintf (stderr, "Cipher: ");
+	  fprintf (stderr, "Cipher, Digest: ");
 	  for (i = 0; algo_table[i].name; i++)
 	    fprintf (stderr, "%s ", algo_table[i].name);
 	  fprintf (stderr, "\n");
 	  return 0;
 
+        case 'j':
+          sigmode = CDK_SIGMODE_DETACHED;
+          break;
+
+        case 'q':
+          show_prefs (user, pub_db);
+          break;
+
         case 'm':
           digest_algid = algo_get_id (digest_algo, 0);
           if (verbose)
-            fprintf (stdout, "Use %s as the message digest (%d)\n",
+            fprintf (stdout, "use %s as the message digest (%d)\n",
                      digest_algo, digest_algid);
           break;
           
 	case 'n':
 	  cipher_algid = algo_get_id (cipher_algo, 1);
 	  if (verbose)
-	    fprintf (stdout, "Use %s for encryption (%d)\n",
+	    fprintf (stdout, "use %s for encryption (%d)\n",
 		     cipher_algo, cipher_algid);
 	  break;
 
         case 'y':
+          cdk_set_log_level (CDK_LOG_DEBUG);
           cdk_handle_set_keydb (hd, pub_db);
           rc = cdk_file_verify (hd, file, NULL);
           if (rc)
             file_error ("verify", file, rc);
           else
-            print_verify_stats (hd);
+            print_verify_stats (hd, 0);
           break;
           
 	case 'u':
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
 	  output = cdk_calloc (1, strlen (file) + 1);
+          if (!output)
+            out_of_core ();
 	  if (strstr (file, ".asc"))
 	    strncpy (output, file, strlen (file) - 4);
 	  else
@@ -809,7 +948,7 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'l':
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
 	  cdk_strlist_add (&klist, user);
 	  do_list_sigs (pub_db, klist);
 	  break;
@@ -824,19 +963,31 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'z':
-	  fprintf (stderr, "minpg: Compression is not available YET!\n");
+          zip_level = atol (compress_level);
+          if (zip_level < 0 || zip_level > 10)
+            {
+              printf ("invalid compression level `%d', changed to 6.\n",
+                      zip_level);
+              zip_level = 6;
+            }
+          cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_COMPRESS,
+                              CDK_COMPRESS_ZIP, zip_level);
 	  break;
 
 	case 'c':
+          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "symencrypt `%s' -> %s\n", file, output);
 	  if (cipher_algid)
-            cdk_handle_set_cipher (hd, cipher_algid);
+            cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_CIPHER,
+                                cipher_algid);
           if (digest_algid)
             {
-              cdk_handle_set_digest (hd, digest_algid);
-              cdk_handle_set_s2k (hd, 3, digest_algid, cipher_algid);
+              cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_DIGEST,
+                                  digest_algid);
+              cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_S2K,
+                                  3, digest_algid, cipher_algid);
             }
           cdk_handle_set_keydb (hd, pub_db);
 	  rc = cdk_file_encrypt (hd, NULL, file, output);
@@ -867,6 +1018,7 @@ main (int argc, char ** argv)
 	  break;
 
 	case 'e':
+        case '@':
 	  if (!remusr)
 	    {
 	      fprintf (stdout, "No recipient was chosen.\n\n");
@@ -874,7 +1026,7 @@ main (int argc, char ** argv)
 	      fgets (u, sizeof u, stdin);
 	      cdk_strlist_add (&remusr, user);
 	    }
-          cdk_set_log_level (CDK_LOG_DEBUG);
+          /*cdk_set_log_level (CDK_LOG_DEBUG);*/
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "encrypt `%s' -> %s\n", file, output);
@@ -882,9 +1034,12 @@ main (int argc, char ** argv)
 	  rc = cdk_file_encrypt (hd, remusr, file, output);
 	  if (rc)
             file_error ("encrypt file", file, rc);
+          if (c == '@')
+            nfiles = 1;
 	  break;
 
 	case 'd':
+        case '$':
           cdk_set_log_level (CDK_LOG_DEBUG);
 	  output = do_remove_ext (file);
 	  if (verbose)
@@ -894,6 +1049,10 @@ main (int argc, char ** argv)
 	  rc = cdk_file_decrypt (hd, file, output);
 	  if (rc)
             file_error ("decrypt file", file, rc);
+          else
+            print_verify_stats (hd, 0);
+          if (c == '$')
+            nfiles = 2;
 	  break;
 
         case 'w':
@@ -912,7 +1071,9 @@ main (int argc, char ** argv)
           if (verbose)
             fprintf (stdout, "sign `%s' -> %s\n", file, output);
           cdk_handle_set_keydb (hd, sec_db);
-          rc = cdk_file_sign (hd, remusr, NULL, file, output, 1, 0);
+          if (sigmode == CDK_SIGMODE_DETACHED)
+            fprintf (stdout, "create a detached signature\n");
+          rc = cdk_file_sign (hd, remusr, NULL, file, output, sigmode, 0);
           if (rc)
             file_error ("sign file", file, rc);
           break;
@@ -925,7 +1086,7 @@ main (int argc, char ** argv)
             }
           output = do_add_ext (file, ".asc");
           cdk_handle_set_keydb (hd, sec_db);
-          rc = cdk_file_clearsign (hd, remusr, file, output);
+          rc = cdk_file_sign (hd, remusr, NULL, file, output, CDK_SIGMODE_CLEAR, 0);
           if (rc)
             file_error ("clearsign file", file, rc);
           break;
@@ -940,13 +1101,42 @@ main (int argc, char ** argv)
           cdk_strlist_add (&klist, "opencdk");
           cdk_handle_set_keydb (hd, pub_db);
           cdk_handle_set_keydb (hd, sec_db);
-          rc = cdk_file_sign (hd, klist, remusr, file, output, 0, 1);
+          rc = cdk_file_sign (hd, klist, remusr, file, output, CDK_SIGMODE_NORMAL, 1);
           if (rc)
             file_error ("sign encrypt", file, rc);
           break;
 	}
     }
 
+  if (nfiles)
+    {
+      const char ** files = poptGetArgs (opt_ctx);
+      /*cdk_handle_control (hd, CDK_CTLF_SET, CDK_CTL_KEYCACHE_FREE, 1);*/
+      while (*files)
+        {
+          if (nfiles == 1)
+            {
+              output = do_add_ext (*files, ".gpg");
+              if (verbose)
+                printf ("encrypt `%s' -> %s\n", *files, output);
+              rc = cdk_file_encrypt (hd, remusr, *files, output);
+            }
+          else if (nfiles == 2)
+            {
+              output = do_remove_ext (*files);
+              if (verbose)
+                printf ("decrypt `%s' -> %s\n", *files, output);
+              rc = cdk_file_decrypt (hd, *files, output);
+            }
+          if (rc)
+              printf ("%s file `%s': %s\n", nfiles==1? "encrypt" : "decrypt",
+                      *files, cdk_strerror (rc));
+          files++;
+          cdk_free (output);
+          output = NULL;
+        }
+    }
+
   cdk_keydb_free (pub_db);
   cdk_keydb_free (sec_db);
   cdk_handle_free (hd);
