@@ -24,6 +24,7 @@
 #include <errno.h>
 #include <time.h>
 #include <stdarg.h>
+#include <assert.h>
 
 #include "opencdk.h"
 
@@ -441,7 +442,7 @@ stream_clearsign (char *file)
   CDK_HD hd;
   CDK_STRLIST locusr = NULL;
   CDK_KEYDB_HD db;
-  int rc;
+  int rc ;
   
   cdk_handle_new (&hd);
   cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
@@ -539,6 +540,7 @@ test_classify_function (void)
 #endif
 
 
+#if 0
 int
 main (int argc, char **argv)
 {
@@ -567,6 +569,169 @@ main (int argc, char **argv)
   
   return 0;
 }
+#endif
 
 
+int
+test_sign (int argc, char ** argv)
+{
+    CDK_HD hd;
+    CDK_KEYDB_HD db;
+    CDK_STRLIST locusr = NULL;
+    char * s = "/home/twoaday/cvs/gnupg.org/opencdk/tests/sec-with-pwd.gpg";
+    const char * test = "Hello World!";
+    unsigned char * out = NULL;
+    size_t olen = 0;
+    int rc;
+    
+    rc = cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, s, strlen (s));
+    assert (!rc);
+    rc = cdk_handle_new (&hd);
+    assert (!rc);
+    cdk_handle_set_keydb (hd, db);
+    cdk_handle_set_armor( hd, 1 );
+    cdk_strlist_add (&locusr, "opencdk");
+    rc = cdk_data_transform (hd, CDK_CRYPTYPE_SIGN, locusr, NULL,
+                             test, strlen (test), &out, &olen,
+                             CDK_SIGMODE_NORMAL);
+    printf ("rc=%d(_%s) out=%p olen=%d\n", rc, cdk_strerror (rc), out, olen);
+    if (!rc)
+        printf ("%s\n", out);
+    
+    cdk_strlist_free (locusr);
+    cdk_handle_free (hd);
+    cdk_keydb_free (db);
+    cdk_free (out);
+    return 0;
+}
+
+
+int
+test_encrypt (int argc, char ** argv)
+{
+    CDK_HD hd;
+    CDK_KEYDB_HD db;
+    CDK_STRLIST remusr = NULL;
+    char * s = "/home/twoaday/.gnupg/pubring.gpg";
+    const char *test = "Hello World!";
+    unsigned char * out = NULL;
+    size_t olen = 0;
+    int rc, cmd = CDK_CRYPTYPE_ENCRYPT;
+
+    rc = cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, s, strlen (s));
+    assert (!rc);
+    rc = cdk_handle_new (&hd);
+    assert (!rc);
+    cdk_handle_set_keydb (hd, db);
+    cdk_handle_set_armor( hd, 1 );
+
+    cdk_strlist_add (&remusr, "opencdk");
+    rc = cdk_data_transform (hd, cmd, NULL, remusr,
+                             test, strlen (test), &out, &olen, 0);
+    printf ("rc=%d out=%p outlen=%d\n", rc, out, olen);
+    if (!rc)
+        printf ("%s\n", out);
+
+    cdk_keydb_free (db);
+    cdk_strlist_free (remusr);
+    cdk_free (out);
+    cdk_handle_free (hd);
+
+    return 0;
+}
+
+
+int
+test_verify (int argc, char ** argv)
+{
+    CDK_HD hd;
+    CDK_KEYDB_HD db;
+    int rc;
+    char * s = "/home/twoaday/.gnupg/pubring.gpg";
+    const char * st =
+        "-----BEGIN PGP MESSAGE-----\n"
+        "Version: OpenPrivacy 0.4.2-cvs\n"
+        "\n"
+        "xA0DAAIRvVcs3MzAfDUByxpiCF9DT05TT0xFPiWDbEhlbGxvIFdvcmxkIcJGBAAR\n"
+        "AgAGBQI+JYNsAAoJEL1XLNzMwHw1YHYAn2e58AeNp1aXc76toHRHpoFkb6ChAJ49\n"
+        "ey3CPJI/sDoy9NPoa3/vBAiYzA==\n"
+        "=jB61\n"
+        "-----END PGP MESSAGE-----\n";
+    unsigned char * out = NULL;
+    size_t olen = 0;
+
+    cdk_set_log_level (CDK_LOG_DEBUG);
+    rc = cdk_handle_new (&hd);
+    assert (!rc);
+    rc = cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, s, strlen (s));
+    assert (!rc);
+    cdk_handle_set_keydb (hd, db);
+
+    rc = cdk_data_transform (hd, CDK_CRYPTYPE_VERIFY, NULL, NULL,
+                             st, strlen (st), &out, &olen, 0);
+    printf ("rc=%d out=%p outlen=%d\n", rc, out, olen );
+    if( !rc )
+        printf ("%s\n", out );
+
+    cdk_free (out);
+    cdk_handle_free (hd);
+    cdk_keydb_free (db);
+    return 0;
+}
+
+        
+int
+test_decrypt (int argc, char ** argv)
+{
+    CDK_HD hd;
+    CDK_KEYDB_HD db;
+    int rc;
+    char * s = "/home/twoaday/cvs/gnupg.org/opencdk/tests/sec-with-pwd.gpg";
+    const char * ct =
+        "-----BEGIN PGP MESSAGE-----\n"
+        "Version: OpenPrivacy 0.4.2-cvs\n"
+        "\n"
+        "wcBOA/ywzzpSYeBnEAP/W2fLuyjEdXoFxIUq6mgxgbH/t9/2PPEHpk+ZX+X79R7I\n"
+        "SHHua/yky6o/4nwYwvXFcnudHjXkN7NqA0qe/qXImN+YnCf9zJFNt31YrI5KGvEz\n"
+        "pG0U89U0+vWtOJmCWPGi8FiR/xF6nMX598dhnV/D0Bqo9EM+0VY209pSSgULf9YD\n"
+        "/0NQGPQaQy8Ls8JYOWd2NRFBfyNO9vTvayyOjEf7XNPc+XUSx6VEACCefc1mBqn+\n"
+        "XW4ZthoAHBNeEBjdWiNvFUsMtiwNI6o3MvO8wJABcuRDT3cqx4vtSBqQpDipXeAX\n"
+        "xxS/mlTvPu9UOURbwtRnh/lG8AshydM3EWHGro+ODT6v0kkBSbmIpS0bvcZx2lz0\n"
+        "t6kJNfo1ZnPzEu1T7nkWs5AQWfQjDnX0AEDaeqi39qYQilaE4sPobJ1HiRVwVrVo\n"
+        "YFDSMaPRxHzaqtwV\n"
+        "=geuH\n"
+        "-----END PGP MESSAGE-----\n";
+    unsigned char * out = NULL;
+    size_t olen = 0;
+
+    rc = cdk_handle_new (&hd);
+    assert (!rc);
+    rc = cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, s, strlen (s));
+    assert (!rc);
+    cdk_handle_set_keydb (hd, db);
+
+    rc = cdk_data_transform (hd, CDK_CRYPTYPE_DECRYPT, NULL, NULL,
+                             ct, strlen (ct), &out, &olen, 0);
+    printf ("rc=%d out=%p outlen=%d\n", rc, out, olen);
+    if (!rc)
+        printf ("%s\n", out);
+    
+    cdk_handle_free (hd);
+    cdk_keydb_free (db);
+    cdk_free (out);
+    return 0;
+}
+
+
+int
+main (int argc, char ** argv)
+{
+    /*test_encrypt (argc, argv);*/
+    /*test_decrypt (argc, argv);*/
+    /*test_sign (argc, argv);*/
+    test_verify (argc, argv);
+    return 0;
+    
+}
+
 
